package org.processmining.mixedparadigm.algorithms.replayer;

import nl.tue.astar.AStarThread;
import nl.tue.astar.Delegate;
import nl.tue.astar.Head;
import nl.tue.astar.Tail;
import nl.tue.astar.Trace;
import nl.tue.storage.compressor.BitMask;

import org.processmining.plugins.astar.petrinet.impl.PHead;
import org.processmining.plugins.astar.petrinet.impl.PRecord;

public class MPRecord extends PRecord {

	public MPRecord(long state, int cost, PRecord predecessor, int logMove, int modelMove, int markingsize,
			int backtrace, BitMask executed) {
		super(state, cost, predecessor, logMove, modelMove, markingsize, backtrace, executed);
	}

	public MPRecord(int cost, PRecord predecessor, int markingsize, int traceLength) {
		super(cost, predecessor, markingsize, traceLength);
	}

	/**
	 * In case of a LogMove only, then logMove>=0, modelMove ==
	 * AStarThread.NOMOVE,
	 * 
	 * In case of a ModelMove only, then logMove == AStarThread.NOMOVE,
	 * modelMove >=0,
	 * 
	 * in case of both log and model move, then logMove>=0, modelMove>=0,
	 * 
	 */
	public MPRecord getNextRecord(Delegate<? extends Head, ? extends Tail> d, Trace trace, Head nextHead, long state,
			int modelMove, int movedEvent, int activity) {

		PILPDelegateAutomata delegate = (PILPDelegateAutomata) d;

		assert !(modelMove != AStarThread.NOMOVE && movedEvent != AStarThread.NOMOVE)
				|| delegate.getActivitiesFor((short) modelMove).contains((short) activity);

		int c = delegate.getCostFor(this.state, (PAutomataHead) nextHead, modelMove, activity);

		BitMask newExecuted;
		if (movedEvent != AStarThread.NOMOVE) {
			newExecuted = executed.clone();
			newExecuted.set(movedEvent, true);
		} else {
			newExecuted = executed;
		}

		MPRecord r = new MPRecord(state, cost + c, this, movedEvent, modelMove, ((PHead) nextHead).getMarking()
				.getNumElts(), backtrace + 1, newExecuted);

		return r;
	}

}
