package org.processmining.mixedparadigm.algorithms.replayer;

import gnu.trove.list.TIntList;

import java.util.Arrays;

import nl.tue.astar.AStarThread;
import nl.tue.astar.Delegate;
import nl.tue.astar.Head;
import nl.tue.astar.Record;
import nl.tue.astar.Tail;
import nl.tue.astar.util.ShortShortMultiset;

import org.deckfour.xes.model.XTrace;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.astar.petrinet.impl.AbstractPDelegate;
import org.processmining.plugins.astar.petrinet.impl.PHead;

public class PAutomataHead extends PHead {

	protected final short[] states; // 4 + 2 * numAutomata

	public static int computeBitsForParikh(short acts, short places) {
		return Math.max(4, (int) (32.0 * acts / (acts + 2 * places)));
	}

	public PAutomataHead(AbstractPDelegate<?> delegate, Marking m, XTrace t, short[] states) {
		super(delegate, m, t);
		this.states = states;

		hashCode = Arrays.hashCode(states) * 31 + PROVIDER.hash(marking, parikh);
	}

	protected PAutomataHead(ShortShortMultiset marking, ShortShortMultiset parikh, short[] states, int hashCode) {
		super(marking, parikh, hashCode);
		this.states = states;
	}

	protected PAutomataHead createHead(ShortShortMultiset marking, ShortShortMultiset parikh, short[] states,
			int hashCode) {
		return new PAutomataHead(marking, parikh, states, hashCode);
	}

	public PAutomataHead getNextHead(Record rec, Delegate<? extends Head, ? extends Tail> d, int modelMove,
			int logMove, int activity) {
		PILPDelegateAutomata delegate = (PILPDelegateAutomata) d;

		final short[] newStates;
		final ShortShortMultiset newMarking;
		if (modelMove != AStarThread.NOMOVE) {
			newStates = new short[states.length];
			System.arraycopy(states, 0, newStates, 0, states.length);
			newMarking = cloneAndUpdateMarkingAndStates(delegate, marking, newStates, (short) modelMove);
		} else {
			newStates = states;
			newMarking = marking;
		}

		final ShortShortMultiset newParikh;
		if (logMove != AStarThread.NOMOVE) {
			newParikh = parikh.clone();
			newParikh.adjustValue((short) activity, (short) -1);
		} else {
			newParikh = parikh;
		}

		return createHead(newMarking, newParikh, newStates,
				Arrays.hashCode(newStates) * 31 + PROVIDER.hash(newMarking, newParikh));
	}

	protected ShortShortMultiset cloneAndUpdateMarkingAndStates(PILPDelegateAutomata delegate,
			ShortShortMultiset marking, short[] states, short modelMove) {
		ShortShortMultiset newMarking = delegate.constructNewMarking(marking, modelMove);

		delegate.updateAutomataStates(states, modelMove);

		return newMarking;
	}

	@Override
	public TIntList getModelMoves(Record rec, Delegate<? extends Head, ? extends Tail> d) {
		return ((PILPDelegateAutomata) d).getEnabledTransitionsChangingMarking(marking, states);
	}

	@Override
	public TIntList getSynchronousMoves(Record rec, Delegate<? extends Head, ? extends Tail> d, TIntList enabled,
			int activity) {
		final PILPDelegateAutomata delegate = (PILPDelegateAutomata) d;
		return delegate.getSyncMoves(marking, states, activity);

	}

	@Override
	public boolean isFinal(Delegate<? extends Head, ? extends Tail> d) {
		PILPDelegateAutomata delegate = (PILPDelegateAutomata) d;
		return parikh.isEmpty() && delegate.isFinal(marking, states);
	}

	@Override
	public boolean equals(Object o) {
		return (o != null) && (o instanceof PHead) && (((PAutomataHead) o).marking.equals(marking))
				&& (((PAutomataHead) o).parikh.equals(parikh) && Arrays.equals(((PAutomataHead) o).states, states));
	}

	@Override
	public String toString() {
		return "[m:" + marking + "<BR/>p:" + parikh + "<BR/>s:" + Arrays.toString(states) + "]";
	}

	public short[] getStates() {
		return states;
	}

}
