package org.processmining.mixedparadigm.algorithms.replayer;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;

import nl.tue.astar.Tail;
import nl.tue.astar.impl.State;
import nl.tue.astar.util.ShortShortMultiset;
import nl.tue.storage.CompressedStore;
import nl.tue.storage.compressor.BitMask;
import nl.tue.storage.impl.SkippableOutputStream;

import org.processmining.plugins.astar.petrinet.impl.PHead;
import org.processmining.plugins.astar.petrinet.impl.PHeadCompressor;

public class PAutomataHeadCompressor<T extends Tail> extends PHeadCompressor<T> {

	private short automata;
	private int bytesAutomata;

	/**
	 * Construct the compressor with a fixed length for all vectors.
	 * 
	 * @param length
	 */
	public PAutomataHeadCompressor(short places, short activities) {
		super(places, activities);
	}

	public void setAutomataCount(short automata) {
		// +1 for the termination flag of the state.
		this.automata = (short) (automata + 1);
		//this.bits = PHead.computeBitsForParikh(activities, places);
		this.bytesAutomata = BitMask.getNumBytes(automata);
		this.maxBytes = 4 + bytesPlaces + places * 2 + bytesActivities + activities * 2 + this.automata * 2
				+ bytesAutomata;
	}

	public PAutomataHead inflate(InputStream stream) throws IOException {
		// skip the hashCode
		int hashCode = readIntFromStream(stream);
		// read the marking
		BitMask mask1 = readMask(stream, places, bytesPlaces);
		ShortShortMultiset marking = inflateContent(stream, BitMask.getIndices(mask1), places);
		// read the vector
		BitMask mask2 = readMask(stream, activities, bytesActivities);
		ShortShortMultiset parikh = inflateContent(stream, BitMask.getIndices(mask2), activities);
		//		System.out.println("r:" + hashCode + "-" + marking + "-" + parikh);
		BitMask mask3 = readMask(stream, automata, bytesAutomata);
		short[] states = inflateContentToArray(stream, BitMask.getIndices(mask3), automata);

		assert Arrays.hashCode(states) * 31 + PAutomataHead.PROVIDER.hash(marking, parikh) == hashCode;

		return new PAutomataHead(marking, parikh, states, hashCode);//, hashCode, bits);
	}

	@Override
	public void deflate(PHead object, OutputStream stream) throws IOException {
		// cache the hashcode for quick lookup
		writeIntToByteArray(stream, object.hashCode());
		// store the marking
		deflate(object.getMarking(), stream, places);
		// store the parikh vector
		deflate(object.getParikhVector(), stream, activities);
		//		System.out.println("w:" + object.hashCode() + "-" + object.getMarking() + "-" + object.getParikhVector());
		deflate(((PAutomataHead) object).getStates(), stream, automata);

	}

	@Override
	protected boolean equalsInflating(State<PHead, T> vector, CompressedStore<State<PHead, T>> store, long l)
			throws IOException {
		InputStream stream = store.getStreamForObject(l);

		int hashCode = readIntFromStream(stream);
		if (hashCode != vector.getHead().hashCode()) {
			return false;
		}

		BitMask mask = readMask(stream, places, bytesPlaces);
		ShortShortMultiset m = vector.getHead().getMarking();
		if (mask.getOnes() != m.size()) {
			return false;
		}
		if (!inflateContent(stream, BitMask.getIndices(mask), places).equals(m)) {
			return false;
		}

		mask = readMask(stream, activities, bytesActivities);
		ShortShortMultiset p = vector.getHead().getParikhVector();
		if (mask.getOnes() != p.size()) {
			return false;
		}
		if (!inflateContent(stream, BitMask.getIndices(mask), activities).equals(p)) {
			return false;
		}

		mask = readMask(stream, automata, bytesAutomata);
		short[] s = ((PAutomataHead) vector.getHead()).getStates();
		short[] sInfl = inflateContentToArray(stream, BitMask.getIndices(mask), automata);
		if (!Arrays.equals(s, sInfl)) {
			return false;
		}

		return true;

	}

	public int getMaxByteCount() {
		return maxBytes;
	}

	@Override
	public void skip(PHead head, SkippableOutputStream out) throws IOException {
		super.skip(head, out);
		// skip states
		int toSkip = bytesAutomata;
		short[] vals = ((PAutomataHead) head).getStates();
		for (int i = vals.length; i-- > 0;) {
			if (vals[i] > 0) {
				toSkip += 2;
			}
		}
		out.skip(toSkip);
	}

}
