package org.processmining.mixedparadigm.algorithms.replayer;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import nl.tue.astar.Head;
import nl.tue.astar.impl.AbstractCompressor;
import nl.tue.astar.impl.memefficient.StorageAwareDelegate;
import nl.tue.astar.impl.memefficient.TailInflater;
import nl.tue.storage.compressor.BitMask;

public class PAutomataTailCompressor extends AbstractCompressor<PAutomataTail> implements TailInflater<PAutomataTail> {

	protected final short places;
	protected final short activities;
	protected final int automata;
	protected final int bytesAutomata;
	protected final int bytesPlaces;
	protected final int bytesActivities;
	//	private final int maxBytes;

	protected int variables;

	public PAutomataTailCompressor(int columns, short places, short activities, short automata) {
		//		this.columns = columns;
		this.places = places;
		this.activities = activities;
		//		this.bytesColumns = BitMask.getNumBytes(columns);
		this.bytesPlaces = BitMask.getNumBytes(places);
		this.bytesActivities = BitMask.getNumBytes(activities);
		//		this.maxBytes = 8 + bytesColumns + columns * 2;
		this.automata = automata + 1;
		this.bytesAutomata = BitMask.getNumBytes(automata);

		this.variables = columns;

	}

	public void deflate(PAutomataTail tail, OutputStream stream) throws IOException {

		short[] vars = tail.getVariables();
		byte[] toWrite = new byte[variables];

		writeBooleanToByteArray(stream, tail.isExactEstimateKnown());
		writeIntToByteArray(stream, tail.getEstimate());

		if (tail.isExactEstimateKnown()) {

			for (int i = 0; i < variables; i++) {
				int val = vars[i] & 0xFFFF;
				byte b = (byte) ((val >> 8) & 0xFF);
				stream.write(b);
				b = (byte) (val & 0xFF);
				stream.write(b);
			}
			//			writeTo(toWrite, vars, 16);

			//			BitMask mask = makeShortListBitMask(columns, tail.getVariables());
			//			stream.write(mask.getBytes());
			//			for (int i = 0; i < columns; i++) {
			//				if (tail.getVariables()[i] > 0) {
			//					writeShortToByteArray(stream, tail.getVariables()[i]);
			//				}
			//			}
		} else {
			stream.write(new byte[2 * variables]);
		}
	}

	public int getMaxByteCount() {
		//		return maxBytes;
		return 1 + 4 + 2 * variables;
	}

	public <H extends Head> int inflateEstimate(StorageAwareDelegate<H, PAutomataTail> delegate, H head,
			InputStream stream) throws IOException {
		skipHead(stream);
		stream.skip(1);
		return readIntFromStream(stream);
	}

	public void skipHead(InputStream stream) throws IOException {
		stream.skip(4);
		BitMask mask = readMask(stream, places, bytesPlaces);
		stream.skip(2 * mask.getOnes());
		mask = readMask(stream, activities, bytesActivities);
		stream.skip(2 * mask.getOnes());
		mask = readMask(stream, automata, bytesAutomata);
		stream.skip(2 * mask.getOnes());

	}

	public PAutomataTail inflate(InputStream stream) throws IOException {
		boolean exact = readBooleanFromStream(stream);
		int est = readIntFromStream(stream);
		short[] vars = new short[variables];
		if (exact) {

			byte[] toRead = new byte[2 * variables];
			stream.read(toRead);
			for (int i = 0, j = 0; i < variables; i++) {
				vars[i] = (short) (toRead[j++] << 8 + toRead[j++]);

			}
		}

		return new PAutomataTail(est, vars, exact);
	}

}
