package org.processmining.mixedparadigm.controller;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.mixedparadigm.models.mixedparadigm.DeclarativeArc;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomataImpl;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.AlternatePrecedence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.AlternateResponse;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.AlternateSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.ChainPrecedence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.ChainResponse;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.ChainSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.CoExistence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.NotChainSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.NotCoExistence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.NotSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.Precedence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.RespondedExistence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.Response;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.Succession;
import org.processmining.mixedparadigm.parameters.DatabaseDeclareParameters;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;


public class DatabaseDeclareController {

	private final DatabaseDeclareParameters parameters;
	private Connection conn;
	private HashMap<String, Integer> declare2;
	private HashMap<List<String>, Integer> declare3;
	private HashMap<List<String>, Integer> declare4;
	private HashMap<List<String>, Integer> declare5;
	private HashMap<List<String>, Integer> declare67;
	private HashMap<List<String>, Integer> declare8;
	private HashMap<List<String>, Integer> declare9;
	
	private HashMap<List<String>, Double> respondedExistence;
	private HashMap<List<String>, Double> response;
	private HashMap<List<String>, Double> alternateResponse;
	private HashMap<List<String>, Double> chainResponse;
	private HashMap<List<String>, Double> precedence;
	private HashMap<List<String>, Double> alternatePrecedence;
	private HashMap<List<String>, Double> chainPrecedence;
	private HashMap<List<String>, Double> coExistence;
	private HashMap<List<String>, Double> notCoExistence;
	private HashMap<List<String>, Double> succession;
	private HashMap<List<String>, Double> notSuccession;
	private HashMap<List<String>, Double> alternateSuccession;
	private HashMap<List<String>, Double> chainSuccession;
	private HashMap<List<String>, Double> notChainSuccession;
	
	private ArrayList<String> activities;
	private HashMap<String, Transition> transitions;
	
	private static PetrinetWithAutomata petrinet;
	private static Marking marking;
	

	public DatabaseDeclareController(DatabaseDeclareParameters parameter) throws SQLException {
		
		this.parameters = parameter;
		
		this.declare2 = new HashMap<String, Integer>();
		this.declare3 = new HashMap<List<String>, Integer>();
		this.declare4 = new HashMap<List<String>, Integer>();
		this.declare5 = new HashMap<List<String>, Integer>();
		this.declare67 = new HashMap<List<String>, Integer>();
		this.declare8 = new HashMap<List<String>, Integer>();
		this.declare9 = new HashMap<List<String>, Integer>();		
		
		this.respondedExistence = new HashMap<List<String>, Double>();	
		this.response = new HashMap<List<String>, Double>();	
		this.alternateResponse = new HashMap<List<String>, Double>();	
		this.chainResponse = new HashMap<List<String>, Double>();	
		this.precedence = new HashMap<List<String>, Double>();	
		this.alternatePrecedence = new HashMap<List<String>, Double>();	
		this.chainPrecedence = new HashMap<List<String>, Double>();	
		this.coExistence = new HashMap<List<String>, Double>();	
		this.notCoExistence = new HashMap<List<String>, Double>();	
		this.succession = new HashMap<List<String>, Double>();	
		this.notSuccession = new HashMap<List<String>, Double>();	
		this.alternateSuccession = new HashMap<List<String>, Double>();	
		this.chainSuccession = new HashMap<List<String>, Double>();	
		this.notChainSuccession = new HashMap<List<String>, Double>();	
		
		this.activities = new ArrayList<String>();
		this.transitions = new HashMap<String,Transition>();
		
		this.petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		this.marking = new Marking();
		
		long startTime1 = System.nanoTime();	
			    
		setupConnection();
		if(parameters.getCreateIS() == true) {
			System.out.println("---initialize and add trigger---");
			initialize();
			addTrigger();
		}
		retrieveDeclares();
		retrieveActivities();
		
		long stopTime1 = System.nanoTime();
	    long elapsedTime1 = stopTime1 - startTime1;
	    
	    // retrieve
	    System.out.println( elapsedTime1/1000000.0);
	    
	    long startTime2 = System.nanoTime();	
		
		countSupport();
		prune(parameters.getThreshold()); 
		
		long stopTime2 = System.nanoTime();
	    long elapsedTime2 = stopTime2 - startTime2;
	    
	    // mining
	    System.out.println(elapsedTime2/1000000.0);

	}
	
	private void setupConnection() throws SQLException {
		java.util.Properties props = new java.util.Properties();
				
		props.put("user", parameters.getUsername()); 
		props.put("password", parameters.getPassword());
		conn = DriverManager.getConnection("jdbc:mysql://" + parameters.getServer() +"/" + parameters.getDatabaseName() + "?useSSL=false&useUnicode=true&useJDBCCompliantTimezoneShift=true&useLegacyDatetimeCode=false&serverTimezone=UTC",props);
	}
	
	private void initialize() {
		String drop = "drop table if exists flatten_log, declare2, declare3, declare4, declare5, declare6_7, declare8, declare9;";
		
		String createFlattenLog= "CREATE TABLE IF NOT EXISTS `flatten_log` ( `log_id` int NOT NULL, `trace_id` int NOT NULL, `event_id` int NOT NULL, `eventclass` varchar(150) NOT NULL, `sequence` int(11), PRIMARY KEY (`log_id`,`trace_id`,`event_id`), KEY `idx_log_id` (`log_id`), KEY `idx_trace_id` (`trace_id`), KEY `idx_event_id` (`event_id`), KEY `idx_eventclass` (`eventclass`), KEY `idx_seq` (`sequence`), KEY `idx_log_trace_ec` (`log_id`, `trace_id`, `eventclass`), FOREIGN KEY (`log_id`) REFERENCES `log` (`id`) ON DELETE CASCADE ON UPDATE CASCADE, FOREIGN KEY (`trace_id`) REFERENCES `trace` (`id`) ON DELETE CASCADE ON UPDATE CASCADE, FOREIGN KEY (`event_id`) REFERENCES `event` (`id`) ON DELETE CASCADE ON UPDATE CASCADE ) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
		String createDeclare2 = "CREATE TABLE IF NOT EXISTS `declare2` ( `id` int NOT NULL, `eventclass` varchar(150) NOT NULL, `freq` int(11) NOT NULL, PRIMARY KEY (`id`, `eventclass`), KEY `idx_eventclass` (`eventclass`), FOREIGN KEY (`id`) REFERENCES `log_properties` (`declare_id`) ON DELETE CASCADE ON UPDATE CASCADE ) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
		String createDeclare3 = "CREATE TABLE IF NOT EXISTS `declare3` ( `id` int NOT NULL, `eventclass1` varchar(150) NOT NULL, `eventclass2` varchar(150) NOT NULL, `freq` int(11) NOT NULL, PRIMARY KEY (`id`, `eventclass1`, `eventclass2`), KEY `idx_eventclass1` (`eventclass1`), KEY `idx_eventclass2` (`eventclass2`), FOREIGN KEY (`id`) REFERENCES `log_properties` (`declare_id`) ON DELETE CASCADE ON UPDATE CASCADE ) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
		String createDeclare4 = "CREATE TABLE IF NOT EXISTS `declare4` ( `id` int NOT NULL, `eventclass1` varchar(150) NOT NULL, `eventclass2` varchar(150) NOT NULL, `freq` int(11) NOT NULL, PRIMARY KEY (`id`, `eventclass1`, `eventclass2`), KEY `idx_eventclass1` (`eventclass1`), KEY `idx_eventclass2` (`eventclass2`), FOREIGN KEY (`id`) REFERENCES `log_properties` (`declare_id`) ON DELETE CASCADE ON UPDATE CASCADE ) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
		String createDeclare5 = "CREATE TABLE IF NOT EXISTS `declare5` ( `id` int NOT NULL, `eventclass1` varchar(150) NOT NULL, `eventclass2` varchar(150) NOT NULL, `freq` int(11) NOT NULL, PRIMARY KEY (`id`, `eventclass1`, `eventclass2`), KEY `idx_eventclass1` (`eventclass1`), KEY `idx_eventclass2` (`eventclass2`), FOREIGN KEY (`id`) REFERENCES `log_properties` (`declare_id`) ON DELETE CASCADE ON UPDATE CASCADE ) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
		String createDeclare67 = "CREATE TABLE IF NOT EXISTS `declare6_7` ( `id` int NOT NULL, `eventclass1` varchar(150) NOT NULL, `eventclass2` varchar(150) NOT NULL, `freq` int(11) NOT NULL, PRIMARY KEY (`id`, `eventclass1`, `eventclass2`), KEY `idx_eventclass1` (`eventclass1`), KEY `idx_eventclass2` (`eventclass2`), FOREIGN KEY (`id`) REFERENCES `log_properties` (`declare_id`) ON DELETE CASCADE ON UPDATE CASCADE ) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
		String createDeclare8 = "CREATE TABLE IF NOT EXISTS `declare8` ( `id` int NOT NULL, `eventclass1` varchar(150) NOT NULL, `eventclass2` varchar(150) NOT NULL, `freq` int(11) NOT NULL, PRIMARY KEY (`id`, `eventclass1`, `eventclass2`), KEY `idx_eventclass1` (`eventclass1`), KEY `idx_eventclass2` (`eventclass2`), FOREIGN KEY (`id`) REFERENCES `log_properties` (`declare_id`) ON DELETE CASCADE ON UPDATE CASCADE ) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
		String createDeclare9 = "CREATE TABLE IF NOT EXISTS `declare9` ( `id` int NOT NULL, `eventclass1` varchar(150) NOT NULL, `eventclass2` varchar(150) NOT NULL, `freq` int(11) NOT NULL, PRIMARY KEY (`id`, `eventclass1`, `eventclass2`), KEY `idx_eventclass1` (`eventclass1`), KEY `idx_eventclass2` (`eventclass2`), FOREIGN KEY (`id`) REFERENCES `log_properties` (`declare_id`) ON DELETE CASCADE ON UPDATE CASCADE ) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
		
		String initFlattenLog= "insert into flatten_log select l.log_id, t.trace_id, e.id, a.value_, t.sequence from log_has_trace as l, trace_has_event as t, event as e, attribute as a where l.trace_id = t.trace_id and t.event_id = e.id and e.attr_id = a.id and a.key_ = 'concept:name';";
		String initDeclare2 = "insert into declare2 select log_id, eventclass, count(*) as freq from flatten_log group by log_id, eventclass;";
		String initDeclare3 = "insert into declare3 select x.log_id, x.eventclass, y.eventclass, count(*) as freq from (select log_id, trace_id, eventclass, sequence as seq from flatten_log) as x, ( (select log_id as logid, trace_id as traceid, eventclass, max(sequence) as seq from flatten_log group by log_id, trace_id, eventclass) union (select distinct f.log_id as logid, f.trace_id as traceid, z.eventclass, \"-1\" as seq from flatten_log as f, (select distinct log_id, eventclass from flatten_log) as z where f.log_id = z.log_id and (f.trace_id, z.eventclass) not in (select trace_id, eventclass from flatten_log) ) ) as y where x.log_id = y.logid and x.trace_id = y.traceid and x.seq > y.seq group by x.log_id, x.eventclass, y.eventclass;";
		String initDeclare4 = "insert into declare4 select x.log_id, x.eventclass, y.eventclass, count(*) as freq from (select log_id, trace_id, eventclass, sequence as seq from flatten_log) as x, ( (select trace_id as traceid, eventclass, min(sequence) as seq from flatten_log group by trace_id, eventclass) union (select distinct f.trace_id as traceid, z.eventclass, \"10000\" as seq from flatten_log as f, (select distinct log_id, eventclass from flatten_log) as z where f.log_id = z.log_id and (f.trace_id, z.eventclass) not in (select trace_id, eventclass from flatten_log) ) ) as y where x.trace_id = y.traceid and x.seq < y.seq group by x.log_id, x.eventclass, y.eventclass;";
		String initDeclare5 = "insert into declare5 select x.log_id, x.eventclass, y.eventclass, count(*) from (select log_id, trace_id, eventclass from flatten_log) as x, (select distinct f.log_id, f.trace_id, z.eventclass from flatten_log as f, (select distinct log_id, eventclass from flatten_log) as z where f.log_id = z.log_id and (f.trace_id, z.eventclass) not in (select trace_id, eventclass from flatten_log) ) as y where x.log_id = y.log_id and x.trace_id = y.trace_id group by x.log_id, x.eventclass, y.eventclass;";
		String initDeclare67 = " insert into declare6_7 select f1.log_id, f1.eventclass, f2.eventclass, count(*) as freq from flatten_log as f1 use index (idx_seq) inner join flatten_log as f2 use index (idx_seq) on f1.trace_id = f2.trace_id and f1.log_id = f2.log_id where f1.sequence = f2.sequence - 1 group by f1.log_id, f1.eventclass, f2.eventclass;";
		String initDeclare8 = "insert into declare8 select f.log_id, f.eventclass1, f.eventclass2, sum(f.freq) from ( select e.log_id as log_id, e.trace_id as trace_id, e.eventclass1 as eventclass1, e.eventclass2 as eventclass2, count(*) - 1 as freq from ( select c.log_id as log_id, c.trace_id as trace_id, c.eventclass1 as eventclass1, c.seq as seq1, c.eventclass2 as eventclass2, min(d.seq) as seq2 from ( select b.log_id as log_id, a.trace_id as trace_id, a.eventclass1 as eventclass1, b.sequence as seq, a.eventclass2 as eventclass2 from ( select f1.trace_id as trace_id, f1.eventclass as eventclass1, f2.eventclass as eventclass2 from (select distinct trace_id, eventclass from flatten_log) as f1, (select distinct trace_id, eventclass from flatten_log) as f2 where f1.trace_id = f2.trace_id and f1.eventclass != f2.eventclass ) as a, ( select log_id, trace_id, eventclass, sequence from flatten_log ) as b where a.trace_id = b.trace_id and a.eventclass1 = b.eventclass ) as c, ( select trace_id, eventclass, sequence as seq from flatten_log ) as d where c.trace_id = d.trace_id and c.eventclass2 = d.eventclass and d.seq > c.seq group by log_id, trace_id, eventclass1, seq1, eventclass2 ) as e group by log_id, trace_id, eventclass1, eventclass2, seq2 ) as f where f.freq > 0 group by log_id, eventclass1, eventclass2;";
		String initDeclare9 = " insert into declare9 select f.log_id, f.eventclass1, f.eventclass2, sum(f.freq) from ( select e.log_id as log_id, e.trace_id as trace_id, e.eventclass1 as eventclass1, e.eventclass2 as eventclass2, count(*) - 1 as freq from ( select c.log_id as log_id, c.trace_id as trace_id, c.eventclass1 as eventclass1, c.seq as seq1, c.eventclass2 as eventclass2, max(d.seq) as seq2 from ( select b.log_id as log_id, a.trace_id as trace_id, a.eventclass1 as eventclass1, b.sequence as seq, a.eventclass2 as eventclass2 from ( select f1.trace_id as trace_id, f1.eventclass as eventclass1, f2.eventclass as eventclass2 from (select distinct trace_id, eventclass from flatten_log) as f1, (select distinct trace_id, eventclass from flatten_log) as f2 where f1.trace_id = f2.trace_id and f1.eventclass != f2.eventclass ) as a, ( select log_id, trace_id, eventclass, sequence from flatten_log ) as b where a.trace_id = b.trace_id and a.eventclass1 = b.eventclass ) as c, ( select trace_id, eventclass, sequence as seq from flatten_log ) as d where c.trace_id = d.trace_id and c.eventclass2 = d.eventclass and d.seq < c.seq group by log_id, trace_id, eventclass1, seq1, eventclass2 ) as e group by log_id, trace_id, eventclass1, eventclass2, seq2 ) as f where f.freq > 0 group by log_id, eventclass1, eventclass2;";
		
		try {			
			Statement st = conn.createStatement();
			st.execute(drop);
			st.execute(createFlattenLog);
			st.execute(createDeclare2);
			st.execute(createDeclare3);			
			st.execute(createDeclare4);
			st.execute(createDeclare5);
			st.execute(createDeclare67);
			st.execute(createDeclare8);
			st.execute(createDeclare9);			
			
			long startTime, stopTime, elapsedTime;
			
			startTime = System.nanoTime();
			st.execute(initFlattenLog);
			stopTime = System.nanoTime();
			elapsedTime = stopTime - startTime;
			System.out.println("init flattenlog " + elapsedTime / 1000000000 + " s");
			
			startTime = System.nanoTime();
			st.execute(initDeclare2);
			stopTime = System.nanoTime();
			elapsedTime = stopTime - startTime;
			System.out.println("init declare2 " + elapsedTime / 1000000000 + " s");
			
			startTime = System.nanoTime();
			st.execute(initDeclare3);
			stopTime = System.nanoTime();
			elapsedTime = stopTime - startTime;
			System.out.println("init declare3 " + elapsedTime / 1000000000 + " s");
			
			startTime = System.nanoTime();
			st.execute(initDeclare4);
			stopTime = System.nanoTime();
			elapsedTime = stopTime - startTime;
			System.out.println("init declare4 " + elapsedTime / 1000000000 + " s");
			
			startTime = System.nanoTime();
			st.execute(initDeclare5);
			stopTime = System.nanoTime();
			elapsedTime = stopTime - startTime;
			System.out.println("init declare5 " + elapsedTime / 1000000000 + " s");
			
			startTime = System.nanoTime();
			st.execute(initDeclare67);
			stopTime = System.nanoTime();
			elapsedTime = stopTime - startTime;
			System.out.println("init declare67 " + elapsedTime / 1000000000 + " s");
			
			startTime = System.nanoTime();
			st.execute(initDeclare8);
			stopTime = System.nanoTime();
			elapsedTime = stopTime - startTime;
			System.out.println("init declare8 " + elapsedTime / 1000000000 + " s");
			
			startTime = System.nanoTime();
			st.execute(initDeclare9);
			stopTime = System.nanoTime();
			elapsedTime = stopTime - startTime;
			System.out.println("init declare9 " + elapsedTime / 1000000000 + " s");
			st.close();
		} catch (SQLException e) {
			e.printStackTrace();
		} 
	}
	
	private void retrieveActivities() {
		String activity = "select eventclass from declare2 where id = \'" + parameters.getLogId() + "\'";
		
		try {
			Statement st = conn.createStatement();
			ResultSet rs = st.executeQuery(activity);
					
			while(rs.next()) {
				activities.add(rs.getString(1));
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
		} 
		
	}
	
	private void addTrigger() {
		String dropTriggerTraceHasEvent = "DROP TRIGGER IF EXISTS trigger_tracehasevent;";
		String createTriggerTraceHasEvent = " CREATE TRIGGER trigger_tracehasevent AFTER INSERT ON trace_has_event FOR EACH ROW BEGIN set @newEvent = new.event_id; set @newTrace = new.trace_id; set @seqNewEC = new.sequence; set @newLog = ''; select log_id from log_has_trace where trace_id = @newTrace into @newLog; set @declareID = @newLog; set @newEC = ''; select a.value_ from event e, attribute a where e.id = @newEvent and e.attr_id = a.id and a.key_ = 'concept:name' into @newEC; set @prevEC = ''; select a.value_ from trace_has_event t, event e, attribute a where t.trace_id = @newTrace and t.sequence = @seqNewEC - 1 and t.event_id = e.id and e.attr_id = a.id and a.key_ = 'concept:name' into @prevEC; if @prevEC <> '' then set @prevfreq = 0; select freq from declare6_7 where id = @declareID and eventclass1 = @prevEC and eventclass2 = @newEC into @prevfreq; set @newfreq = @prevfreq + 1; replace into declare6_7 values (@declareID, @prevEC, @newEC, @newfreq); end if; set @exist = null; select eventclass from declare2 where id = @newLog and eventclass = @newEC into @exist; if @exist is null then BLOCK0: begin declare doneBlock0 int default false; declare act varchar(150); declare cursorAct cursor for select distinct eventclass from declare2 where id = @newLog; declare continue handler for not found set doneBlock0 = true; open cursorAct; loop_act: loop set doneBlock0 = false; fetch cursorAct into act; if doneBlock0 then leave loop_act; end if; if act <> @newEC then set @occ = 0; select freq from declare2 where id = @declareID and eventclass = act into @occ; set @prevfreq = 0; select freq from declare3 where id = @declareID and eventclass1 = act and eventclass2 = @newEC into @prevfreq; set @newfreq = @prevfreq + @occ; replace into declare3 values (@declareID, act, @newEC, @newfreq); set @prevfreq = 0; select freq from declare4 where id = @declareID and eventclass1 = act and eventclass2 = @newEC into @prevfreq; set @newfreq = @prevfreq + @occ; replace into declare4 values (@declareID, act, @newEC, @newfreq); set @prevfreq = 0; select freq from declare5 where id = @declareID and eventclass1 = act and eventclass2 = @newEC into @prevfreq; set @newfreq = @prevfreq + @occ; replace into declare5 values (@declareID, act, @newEC, @newfreq); end if; end loop loop_act; close cursorAct; end BLOCK0; end if; set @prevfreq = 0; select freq from declare2 where id = @declareID and eventclass = @newEC into @prevfreq; set @newfreq = @prevfreq + 1; replace into declare2 values (@declareID, @newEC, @newfreq); BLOCK1: begin declare doneBlock1 int default false; declare otherEC varchar(150); declare cursorEC cursor for select distinct eventclass from declare2 where id = @newLog; declare continue handler for not found set doneBlock1 = true; open cursorEC; loop_ec: loop set doneBlock1 = false; fetch cursorEC into otherEC; if doneBlock1 then leave loop_ec; end if; if @newEC <> otherEC then set @occEC = 0; select freq from declare_all where id = @declareID and trace_id = @newTrace and eventclass1 = @newEC and eventclass2 = @newEC into @occEC; set @occOtherEC = 0; select freq from declare_all where id = @declareID and trace_id = @newTrace and eventclass1 = otherEC and eventclass2 = otherEC into @occOtherEC; set @freqRow = 0; select freq from declare_all where id = @declareID and trace_id = @newTrace and eventclass1 = @newEC and eventclass2 = otherEC into @freqRow; if @freqRow is null then set @newfreq = @occEC + 1; insert into declare_all values (@declareID, @newTrace, @newEC, otherEC, @newfreq); else set @newfreq = @freqRow + 1; replace into declare_all values (@declareID, @newTrace, @newEC, otherEC, @newfreq); end if; if @occOtherEC >= 1 and @freqRow >= 1 then set @prevfreq = 0; select freq from declare9 where id = @declareID and eventclass1 = @newEC and eventclass2 = otherEC into @prevfreq; set @newfreq = @prevfreq + 1; replace into declare9 values (@declareID, @newEC, otherEC, @newfreq); end if; set @prevfreq = 0; select freq from declare3 where id = @declareID and eventclass1 = @newEC and eventclass2 = otherEC into @prevfreq; set @newfreq = @prevfreq + 1; replace into declare3 values (@declareID, @newEC, otherEC, @newfreq); if @occOtherEC = 0 then set @prevfreq = 0; select freq from declare4 where id = @declareID and eventclass1 = @newEC and eventclass2 = otherEC into @prevfreq; set @newfreq = @prevfreq + 1; replace into declare4 values (@declareID, @newEC, otherEC, @newfreq); set @prevfreq = 0; select freq from declare5 where id = @declareID and eventclass1 = @newEC and eventclass2 = otherEC into @prevfreq; set @newfreq = @prevfreq + 1; replace into declare5 values (@declareID, @newEC, otherEC, @newfreq); end if; set @freqCol = null; select freq from declare_all where id = @declareID and trace_id = @newTrace and eventclass1 = otherEC and eventclass2 = @newEC into @freqCol; if @freqCol is null then set @freqCol = @occOtherEC; end if; replace into declare_all values (@declareID, @newTrace, otherEC, @newEC, 0); set @prevfreq = 0; select freq from declare3 where id = @declareID and eventclass1 = otherEC and eventclass2 = @newEC into @prevfreq; set @newfreq = @prevfreq - @freqCol; if @newfreq = 0 then delete from declare3 where id = @declareID and eventclass1 = otherEC and eventclass2 = @newEC; else replace into declare3 values (@declareID, otherEC, @newEC, @newfreq); end if; if @occEC + 1 = 1 then set @prevfreq = 0; select freq from declare5 where id = @declareID and eventclass1 = otherEC and eventclass2 = @newEC into @prevfreq; set @newfreq = @prevfreq - @freqCol; if @newfreq = 0 then delete from declare5 where id = @declareID and eventclass1 = otherEC and eventclass2 = @newEC; else replace into declare5 values (@declareID, otherEC, @newEC, @newfreq); end if; end if; set @freqColMinOne = @freqCol - 1; if @freqColMinOne > 0 then set @prevfreq = 0; select freq from declare8 where id = @declareID and eventclass1 = otherEC and eventclass2 = @newEC into @prevfreq; set @newfreq = @prevfreq + @freqColMinOne; replace into declare8 values (@declareID, otherEC, @newEC, @newfreq); end if; end if; end loop loop_ec; close cursorEC; end BLOCK1; set @prevfreq = 0; select freq from declare_all where id = @declareID and trace_id = @newTrace and eventclass1 = @newEC and eventclass2 = @newEC into @prevfreq; set @newfreq = @prevfreq + 1; replace into declare_all values (@declareID, @newTrace, @newEC, @newEC, @newfreq); END";
		
		try {
			Statement st = conn.createStatement();
			st.execute(dropTriggerTraceHasEvent);
			st.execute(createTriggerTraceHasEvent);
			
			st.close();
		} catch (SQLException e) {
			e.printStackTrace();
		} 
	}
	
	private void retrieveDeclares() {
		String logID = parameters.getLogId();
		String retrieveDeclare2 = "select * from declare2 where id = \'" + logID + "\'";
		String retrieveDeclare3 = "select * from declare3 where id = \'" + logID + "\'";
		String retrieveDeclare4 = "select * from declare4 where id = \'" + logID + "\'";
		String retrieveDeclare5 = "select * from declare5 where id = \'" + logID + "\'";
		String retrieveDeclare67 = "select * from declare6_7 where id = \'" + logID + "\'";
		String retrieveDeclare8 = "select * from declare8 where id = \'" + logID + "\'";
		String retrieveDeclare9 = "select * from declare9 where id = \'" + logID + "\'";
		
		try {
			Statement st = conn.createStatement();
			ResultSet rs;
					
			// declare 2
			rs = st.executeQuery(retrieveDeclare2);
			
			while(rs.next()) {
				String eventclass = rs.getString(2);
				int freq = rs.getInt(3);
				
				if(freq > 0) declare2.put(eventclass, freq);
			}
			
			// declare 3
			rs = st.executeQuery(retrieveDeclare3);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(2);
				String eventclass2 = rs.getString(3);
				int freq = rs.getInt(4);				
					
				if(freq > 0) declare3.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			// declare 4
			rs = st.executeQuery(retrieveDeclare4);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(2);
				String eventclass2 = rs.getString(3);
				int freq = rs.getInt(4);				
					
				if(freq > 0) declare4.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			// declare 5
			rs = st.executeQuery(retrieveDeclare5);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(2);
				String eventclass2 = rs.getString(3);
				int freq = rs.getInt(4);				
					
				if(freq > 0) declare5.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			// declare67
			rs = st.executeQuery(retrieveDeclare67);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(2);
				String eventclass2 = rs.getString(3);
				int freq = rs.getInt(4);				
					
				if(freq > 0) declare67.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			// declare 8
			rs = st.executeQuery(retrieveDeclare8);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(2);
				String eventclass2 = rs.getString(3);
				int freq = rs.getInt(4);				
					
				if(freq > 0) declare8.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			// declare 9
			rs = st.executeQuery(retrieveDeclare9);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(2);
				String eventclass2 = rs.getString(3);
				int freq = rs.getInt(4);				
					
				if(freq > 0) declare9.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			st.close();
		} catch (SQLException e) {
			e.printStackTrace();
		} 
	}
	
	private void countSupport() {
		for(int i = 0; i < activities.size(); i++) {
			String a = activities.get(i);
			for(int j = 0; j < activities.size(); j++) {
				if(i != j) {
					String b = activities.get(j);
					
					List<String> sourceTarget = Arrays.asList(a,b);
					List<String> targetSource = Arrays.asList(b,a);			
				
					Integer dec2 = (declare2.get(a) == null) ? 0 : declare2.get(a);
					Integer dec2Rev = (declare2.get(b) == null) ? 0 : declare2.get(b);
					Integer sumDec2 = dec2 + dec2Rev;
					Integer dec3 = (declare3.get(sourceTarget) == null) ? 0 : declare3.get(sourceTarget);
					Integer dec4 = (declare4.get(sourceTarget) == null) ? 0 : declare4.get(sourceTarget);
					Integer dec4Rev = (declare4.get(targetSource) == null) ? 0 : declare4.get(targetSource);
					Integer dec5 = (declare5.get(sourceTarget) == null) ? 0 : declare5.get(sourceTarget);
					Integer dec5Rev = (declare5.get(targetSource) == null) ? 0 : declare5.get(targetSource);
					Integer dec6 = (declare67.get(sourceTarget) == null) ? 0 : declare67.get(sourceTarget);
					Integer dec7 = (declare67.get(targetSource) == null) ? 0 : declare67.get(targetSource);
					Integer dec7Rev = dec6;
					Integer dec8 = (declare8.get(sourceTarget) == null) ? 0 : declare8.get(sourceTarget);
					Integer dec9 = (declare9.get(sourceTarget) == null) ? 0 : declare9.get(sourceTarget);
					Integer dec9Rev = (declare9.get(targetSource) == null) ? 0 : declare9.get(targetSource);
					
					if(dec2 != 0) {
						Double re = 1.0 - ((double) dec5/dec2);
						if(re>0.0) respondedExistence.put(sourceTarget, re);		
						
						Double r = 1.0 - ((double) dec3/dec2);
						if(r>0.0) response.put(sourceTarget, r);
						
						Double ar = 1.0 - ((double) (dec3+dec8)/dec2);
						if(ar>0.0) alternateResponse.put(sourceTarget, ar);
						
						Double cr = (double) (dec6/dec2);
						if(cr>0.0) chainResponse.put(sourceTarget, cr);
						
						Double p = 1.0 - ((double) dec4/dec2);
						if(p>0.0) precedence.put(targetSource, p);
						
						Double ap = 1.0 - ((double) (dec4+dec9)/dec2);
						if(ap>0.0) alternatePrecedence.put(targetSource, ap);
						
						Double cp = (double) dec7/dec2;
						if(cp>0.0) chainPrecedence.put(targetSource, cp);
					}
					
					if(sumDec2 != 0) {
						Double nce = (double) (dec5 + dec5Rev)/sumDec2;
						Double ce = 1.0 - nce;
						if(ce>0.0) coExistence.put(sourceTarget, ce);
						if(nce>0.0) notCoExistence.put(sourceTarget, nce);
						
						Double ns = (double) (dec3 + dec4Rev)/sumDec2;
						Double s = 1.0 - ns;
						if(s>0.0) succession.put(sourceTarget, s);
						if(ns>0.0) notSuccession.put(sourceTarget, ns);
						
						Double as = 1.0 - ((double) (dec8 + dec3 + dec9Rev + dec4Rev)/sumDec2);
						if(as>0.0) alternateSuccession.put(sourceTarget, as);
						
						 Double cs = (double) (dec6 + dec7Rev)/sumDec2;
						 Double ncs = 1.0 - cs;
						 if(cs>0.0) chainSuccession.put(sourceTarget, cs);
						 if(ncs>0.0) notChainSuccession.put(sourceTarget, ncs);
					} 
				}
			}
		}
	}
	
	private void prune(double threshold) {		
		
		for(int i = 0; i < activities.size(); i++) {
			String act = activities.get(i);
			Transition t = petrinet.addTransition(act);
			transitions.put(act, t);
		}
		
		for(int i = 0; i < activities.size(); i++) {
			String a = activities.get(i);
			for(int j = 0; j < activities.size(); j++) {
				String b = activities.get(j);
				
				List<String> sourceTarget = Arrays.asList(a,b);
				List<String> targetSource = Arrays.asList(b,a);					
				
				Double re = (respondedExistence.get(sourceTarget) == null) ? 0 : respondedExistence.get(sourceTarget);			
				Double r = (response.get(sourceTarget) == null) ? 0 : response.get(sourceTarget);		
				Double ar = (alternateResponse.get(sourceTarget) == null) ? 0 : alternateResponse.get(sourceTarget);		
				Double cr = (chainResponse.get(sourceTarget) == null) ? 0 : chainResponse.get(sourceTarget);		
				Double pR = (precedence.get(targetSource) == null) ? 0 : precedence.get(targetSource);		
				Double apR = (alternatePrecedence.get(targetSource) == null) ? 0 : alternatePrecedence.get(targetSource);		
				Double cpR = (chainPrecedence.get(targetSource) == null) ? 0 : chainPrecedence.get(targetSource);		
				
				boolean existRespondedExistence = false;
				
				// chain response
				if(cr.equals(ar) && cr >= threshold) {
					existRespondedExistence = true;
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new ChainResponse());
				// alternate response
				} else if(ar.equals(r) && ar >= threshold) {
					existRespondedExistence = true;
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new AlternateResponse());
				// response
				} else if(r.equals(re) && r >= threshold) {
					existRespondedExistence = true;
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new Response());
				}
				
				// chain precedence
				if(cpR != null && cpR.equals(apR) && cpR >= threshold) {
					existRespondedExistence = true;
					petrinet.addConstraint(transitions.get(b), transitions.get(a), new ChainPrecedence());
				// alternate precedence
				} else if(apR != null && apR.equals(pR) && apR >= threshold) {
					existRespondedExistence = true;
					petrinet.addConstraint(transitions.get(b), transitions.get(a), new AlternatePrecedence());
				// precedence
				} else if(pR != null && pR.equals(re) && pR >= threshold) {
					existRespondedExistence = true;
					petrinet.addConstraint(transitions.get(b), transitions.get(a), new Precedence());
				} 
				
				// responded existence
				if(re != null && !existRespondedExistence && re >= threshold) {
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new RespondedExistence());
				}
			} 
		}  
	
		 // MUTUAL RELATION		
		for(int i = 0; i < activities.size(); i++) {
			String a = activities.get(i);
			for(int j = 0; j < activities.size(); j++) {
				String b = activities.get(j);
				
				List<String> sourceTarget = Arrays.asList(a,b);
				List<String> targetSource = Arrays.asList(b,a);	
				
				Double re = (respondedExistence.get(sourceTarget) == null) ? 0 : respondedExistence.get(sourceTarget);			
				Double r = (response.get(sourceTarget) == null) ? 0 : response.get(sourceTarget);	
				Double ar = (alternateResponse.get(sourceTarget) == null) ? 0 : alternateResponse.get(sourceTarget);	
				Double cr = (chainResponse.get(sourceTarget) == null) ? 0 : chainResponse.get(sourceTarget);	
				Double p = (precedence.get(sourceTarget) == null) ? 0 : precedence.get(sourceTarget);	
				Double ap = (alternatePrecedence.get(sourceTarget) == null) ? 0 : alternatePrecedence.get(sourceTarget);	
				Double cp = (chainPrecedence.get(sourceTarget) == null) ? 0 : chainPrecedence.get(sourceTarget);	
				Double ce = (coExistence.get(sourceTarget) == null) ? 0 : coExistence.get(sourceTarget);	
				Double reR = (respondedExistence.get(targetSource) == null) ? 0 : respondedExistence.get(targetSource);	
				Double s = (succession.get(sourceTarget) == null) ? 0 : succession.get(sourceTarget);	
				Double as = (alternateSuccession.get(sourceTarget) == null) ? 0 : alternateSuccession.get(sourceTarget);	
				Double cs = (chainSuccession.get(sourceTarget) == null) ? 0 : chainSuccession.get(sourceTarget);	
				
				// chain succession
				if(cs.equals(cr) && cs.equals(cp) && cs >= threshold) {
					DeclarativeArc arc1 = new DeclarativeArc(transitions.get(a), transitions.get(b), new ChainResponse());
					DeclarativeArc arc2 = new DeclarativeArc(transitions.get(a), transitions.get(b), new ChainPrecedence());
					petrinet.removeConstraint(arc1);
					petrinet.removeConstraint(arc2);
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new ChainSuccession());
				// alternate succession
				} else if(as.equals(ar) && as.equals(ap) && as >= threshold) {
					DeclarativeArc arc1 = new DeclarativeArc(transitions.get(a), transitions.get(b), new AlternateResponse());
					DeclarativeArc arc2 = new DeclarativeArc(transitions.get(a), transitions.get(b), new AlternatePrecedence());
					petrinet.removeConstraint(arc1);
					petrinet.removeConstraint(arc2);
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new AlternateSuccession());
				// succession
				} else if(s.equals(r) && s.equals(p) && s >= threshold) {
					DeclarativeArc arc1 = new DeclarativeArc(transitions.get(a), transitions.get(b), new Response());
					DeclarativeArc arc2 = new DeclarativeArc(transitions.get(a), transitions.get(b), new Precedence());
					petrinet.removeConstraint(arc1);
					petrinet.removeConstraint(arc2);
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new Succession());
				// coExistence
				} else if(ce.equals(re) && ce.equals(reR) && ce >= threshold) {
					DeclarativeArc arc1 = new DeclarativeArc(transitions.get(a), transitions.get(b), new RespondedExistence());
					DeclarativeArc arc2 = new DeclarativeArc(transitions.get(b), transitions.get(a), new RespondedExistence());
					petrinet.removeConstraint(arc1);
					petrinet.removeConstraint(arc2);
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new CoExistence());
				}
			}
		}
		
		// NEGATIVE RELATION
		for(int i = 0; i < activities.size(); i++) {
			String a = activities.get(i);
			for(int j = 0; j < activities.size(); j++) {
				String b = activities.get(j);
				
				List<String> sourceTarget = Arrays.asList(a,b);			
		
				Double ce = (coExistence.get(sourceTarget) == null) ? 0 : coExistence.get(sourceTarget);
				Double s = (succession.get(sourceTarget) == null) ? 0 : succession.get(sourceTarget);
				Double cs = (chainSuccession.get(sourceTarget) == null) ? 0 : chainSuccession.get(sourceTarget);
				Double ncs = (notChainSuccession.get(sourceTarget) == null) ? 0 : notChainSuccession.get(sourceTarget);
				Double ns = (notSuccession.get(sourceTarget) == null) ? 0 : notSuccession.get(sourceTarget);
				Double nce = (notCoExistence.get(sourceTarget) == null) ? 0 : notCoExistence.get(sourceTarget);
				
				// not co existence
				if(ce <= nce && nce >= threshold) {
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new NotCoExistence());
				// not succession
				} else if(s <= ns && ns >= threshold) {
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new NotSuccession());
				 //  not chain succession
				} else if(cs <= ncs && ncs >= threshold) {
					petrinet.addConstraint(transitions.get(a), transitions.get(b), new NotChainSuccession());
				}
			}		
		}
	}

	public DatabaseDeclareParameters getParameters() {
		return parameters;
	}
	
	public Object[] printProcessModel(PluginContext context) {
		Collection c = petrinet.getConstraints();
		
		Iterator it = c.iterator();
		
		while(it.hasNext()) {
			System.out.println(it.next());
		}
		
		context.addConnection(new InitialMarkingConnection(petrinet, marking));
		return new Object[] {petrinet, marking};
	}
	
	public void printProcessModel() {		
		Collection c = petrinet.getConstraints();
		
		Iterator it = c.iterator();
		
		while(it.hasNext()) {
			Object s = it.next();
			System.out.println(s);
		}
		
	}
}
