package org.processmining.mixedparadigm.models.mixedparadigm;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.processmining.mixedparadigm.models.mixedparadigm.constraints.ConstraintType;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.ExpandableSubNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.AbstractResetInhibitorNet;

public abstract class AbstractPetrinetWithAutomata extends AbstractResetInhibitorNet {

	protected final Set<DeclarativeArc> constraints = new HashSet<>();

	public AbstractPetrinetWithAutomata(boolean allowsReset, boolean allowsInhibitors) {
		super(allowsReset, allowsInhibitors);
	}

	public synchronized DeclarativeArc addConstraint(Transition from, ConstraintType.Unary type) {
		return addConstraint(from, type, null);
	}

	public synchronized DeclarativeArc addConstraint(Transition from, ConstraintType.Unary type, ExpandableSubNet parent) {
		return addConstraint(from, from, type, parent);
	}

	public synchronized DeclarativeArc addConstraint(Transition from, Transition to, ConstraintType type) {
		return addConstraint(from, to, type, null);
	}

	public synchronized DeclarativeArc addConstraint(Collection<? extends Transition> from,
			Collection<? extends Transition> to, ConstraintType.Branched type) {
		return this.addConstraint(from, to, type, null);
	}

	public synchronized DeclarativeArc addConstraint(Collection<? extends Transition> from,
			Collection<? extends Transition> to, ConstraintType.Branched type, ExpandableSubNet parent) {
		checkAddEdge(from, to);
		Iterator<? extends Transition> fromIt = from.iterator();
		Iterator<? extends Transition> toIt = from.iterator();

		DeclarativeArc a = new DeclarativeArc(fromIt.next(), toIt.next(), type);
		while (fromIt.hasNext()) {
			a.addAdditionalSource(fromIt.next());
		}
		while (toIt.hasNext()) {
			a.addAdditionalTarget(toIt.next());
		}
		if (constraints.add(a)) {
			graphElementAdded(a);
			return a;
		} else {
			for (DeclarativeArc existing : constraints) {
				if (existing.equals(a)) {
					return existing;
				}
			}
		}
		assert (false);
		return null;

	}

	protected void checkAddEdge(Collection<? extends Transition> source, Collection<? extends Transition> target) {
		Collection<?> nodes = getNodes();
		if (!nodes.containsAll(source) && !nodes.containsAll(target)) {
			throw new IllegalArgumentException("Cannot add an arc between " + source.toString() + " and "
					+ target.toString() + ", since one of these nodes is not in the graph.");
		}
		if (source.isEmpty() || target.isEmpty()) {
			throw new IllegalArgumentException("Cannot add an arc between " + source.toString() + " and "
					+ target.toString() + ", since one of these sets is empty.");
		}
	}

	public synchronized DeclarativeArc addConstraint(Transition from, Transition to, ConstraintType type,
			ExpandableSubNet parent) {
		checkAddEdge(from, to);
		DeclarativeArc a = new DeclarativeArc(from, to, type);
		if (constraints.add(a)) {
			graphElementAdded(a);
			return a;
		} else {
			for (DeclarativeArc existing : constraints) {
				if (existing.equals(a)) {
					return existing;
				}
			}
		}
		assert (false);
		return null;
	}

	public synchronized DeclarativeArc removeConstraint(DeclarativeArc arc) {
		for (DeclarativeArc a : constraints) {
			if (a.getSources().equals(arc.getSources()) && a.getTargets().equals(arc.getTargets()) && a.getType().getName().equals(arc.getType().getName())) {
				constraints.remove(a);
				graphElementRemoved(a);
				return a;
			}
		}
		return null;
	}

	public Collection<DeclarativeArc> getConstraints() {
		return Collections.unmodifiableCollection(constraints);
	}

	public synchronized Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> getEdges() {
		Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> set = super.getEdges();
		set.addAll(constraints);
		return set;
	}

}
