package org.processmining.mixedparadigm.models.mixedparadigm;

import java.util.ArrayList;
import java.util.Collection;

import org.processmining.mixedparadigm.models.mixedparadigm.constraints.ActivationMode;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.ConstraintType;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.DirectedGraphEdge.MultipleSources;
import org.processmining.models.graphbased.directed.DirectedGraphEdge.MultipleTargets;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.elements.ExpandableSubNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class DeclarativeArc extends PetrinetEdge<Transition, Transition> implements MultipleSources<Transition>,
		MultipleTargets<Transition> {

	private final ConstraintType type;

	private final Collection<Transition> sources = new ArrayList<>(2);
	private final Collection<Transition> targets = new ArrayList<>(2);

	private final ActivationMode activationMode;

	public DeclarativeArc(Transition source, Transition target, ConstraintType type) {
		this(source, target, type, null);
	}

	public DeclarativeArc(Transition source, Transition target, ConstraintType type, ExpandableSubNet parent) {
		super(parent, source, target, type.getName());
		this.type = type;
		this.activationMode = type.getActivationMode();

		Graphics graphics = type.getGraphics();

		getAttributeMap().put(AttributeMap.EDGESTART, graphics.getStartArrow());
		getAttributeMap().put(AttributeMap.EDGESTARTFILLED, true);

		getAttributeMap().put(AttributeMap.EDGEMIDDLE, graphics.getMiddleArrow());
		getAttributeMap().put(AttributeMap.EDGEMIDDLEFILLED, graphics.getMiddleArrowFilled());

		getAttributeMap().put(AttributeMap.EDGEEND, graphics.getEndArrow());
		getAttributeMap().put(AttributeMap.EDGEENDFILLED, true);

		getAttributeMap().put(AttributeMap.LABEL, type.getLabel());
		if (graphics.isShowLabel()) {
			getAttributeMap().put(AttributeMap.SHOWLABEL, true);
			getAttributeMap().put(AttributeMap.LABELALONGEDGE, true);
		}
		getAttributeMap().put(AttributeMap.NUMLINES, graphics.getNumberOfLines());

		getAttributeMap().put(AttributeMap.DASHPATTERN, new float[] { 7f, 3f });

		sources.add(source);
		targets.add(target);
	}

	public ConstraintType getType() {
		return type;
	}

	public Collection<Transition> getTargets() {
		return targets;
	}

	public Collection<Transition> getSources() {
		return sources;
	}

	/**
	 * This method should be called only after construction of the object and
	 * before using it as new source nodes influence the hashCode
	 * 
	 * @param t
	 */
	public void addAdditionalSource(Transition t) {
		sources.add(t);
	}

	/**
	 * This method should be called only after construction of the object and
	 * before using it as new target nodes influence the hashCode
	 * 
	 * @param t
	 */
	public void addAdditionalTarget(Transition t) {
		targets.add(t);
	}

	public int hashCode() {
		// Hashcode based on source and target, which
		// respects contract that this.equals(o) implies
		// this.hashCode()==o.hashCode()
		return sources.hashCode() * 31 + targets.hashCode();
	}

	public boolean equals(Object o) {
		if (!(this.getClass().equals(o.getClass()))) {
			return false;
		}
		DeclarativeArc edge = (DeclarativeArc) o;

		return edge.sources.equals(sources) && edge.targets.equals(targets) && edge.type.equals(type);

	}

	public String toString() {
		return getLabel() + " " + sources.toString() + " -> " + targets.toString();
	}

	public ActivationMode getActivationMode() {
		return activationMode;
	}
}
