package org.processmining.mixedparadigm.models.mixedparadigm.constraints.branched;

import gnu.trove.list.TShortList;
import gnu.trove.list.array.TShortArrayList;
import gnu.trove.map.TObjectShortMap;
import gnu.trove.map.hash.TObjectShortHashMap;
import gnu.trove.set.TShortSet;
import gnu.trove.set.hash.TShortHashSet;

import java.util.Collection;

import org.processmining.mixedparadigm.models.dfa.DeterministicFiniteAutomaton;
import org.processmining.mixedparadigm.models.dfa.GeneralDeterministicFiniteAutomaton;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.ActivationMode;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.ConstraintType;
import org.processmining.models.graphbased.AttributeMap.ArrowType;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class ExclusiveChoice extends ConstraintType.Branched {

	private short n;

	public ExclusiveChoice(short n) {
		super(ActivationMode.GLOBAL);
		this.n = n;
		graphics.setMiddleArrow(ArrowType.ARROWTYPE_DIAMOND);
		graphics.setMiddleArrowFilled(true);
		graphics.setShowLabel(true);
	}

	public DeterministicFiniteAutomaton getAutomaton(TObjectShortMap<Transition> trans2int,
			Collection<? extends Transition> from, Collection<? extends Transition> to) {
		TShortList trans = new TShortArrayList();
		for (Transition t : from) {
			trans.add(trans2int.get(t));
		}
		for (Transition t : to) {
			trans.add(trans2int.get(t));
		}
		DeterministicFiniteAutomaton dfa = new GeneralDeterministicFiniteAutomaton(getName(), trans2int.size());

		short s0 = dfa.addState(true, DeterministicFiniteAutomaton.TV, true);
		TObjectShortMap<TShortSet> stateSet = new TObjectShortHashMap<>(10, 0.7f, Short.MIN_VALUE);
		TShortSet initial = new TShortHashSet();
		stateSet.put(initial, s0);

		addStates(dfa, trans, initial, stateSet);

		return dfa;
	}

	private short addStates(DeterministicFiniteAutomaton dfa, TShortList trans, TShortSet executed,
			TObjectShortMap<TShortSet> stateSet) {
		TShortHashSet newExecuted = new TShortHashSet(executed);
		short s = stateSet.get(executed);
		if (s == stateSet.getNoEntryValue()) {
			s = dfa.addState(false, executed.size() == n ? DeterministicFiniteAutomaton.PS
					: DeterministicFiniteAutomaton.TV, true);
			stateSet.put(newExecuted, s);
		}
		for (int i = 0; i < trans.size(); i++) {
			short t = trans.get(i);
			if (!executed.contains(t)) {
				// Cannot execute transition t in state s since it was not on the path coming 
				// into this state (if non-final state, then the transition t is added later
				// to a new state
				dfa.removeTransition(s, t);
			}
		}
		if (executed.size() == n) {
			return s;
		}

		for (int i = 0; i < trans.size(); i++) {
			short t = trans.get(i);
			if (newExecuted.add(t)) {
				short toState = addStates(dfa, trans, newExecuted, stateSet);
				dfa.addTransition(s, toState, t);
				newExecuted = new TShortHashSet(executed);
			}
		}
		return s;

	}

	public String getName() {
		return EXCLUSIVECHOICE_ + n;
	}

	public String getLabel() {
		return n + " of N";
	}

}
