package org.processmining.mixedparadigm.parameters;

import java.util.Collection;

import javax.swing.JComponent;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.mixedparadigm.models.mixedparadigm.DeclarativeArc;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayer.algorithms.costbasedcomplete.CostBasedCompleteParamProvider;

public class MPReplayParameterProvider extends CostBasedCompleteParamProvider {

	protected final Collection<DeclarativeArc> constraints;

	public MPReplayParameterProvider(PluginContext context, PetrinetWithAutomata net, XLog log,
			TransEvClassMapping mapping) {
		super(context, net, log, mapping);
		constraints = net.getConstraints();

	}

	public MPReplayParameters constructReplayParameter(JComponent ui) {
		if (ui instanceof MPReplayParametersUI) {
			MPReplayParametersUI hui = (MPReplayParametersUI) ui;

			MPReplayParameters paramObj = new MPReplayParameters(hui.getMapEvClassToCost(), hui.getTransitionWeight());
			paramObj.setMapSync2Cost(hui.getSyncCost());
			paramObj.setMaxNumOfStates(hui.getMaxNumOfStates());
			paramObj.setInitialMarking(initMarking);
			paramObj.setFinalMarkings(finalMarkings);
			paramObj.setUsePartialOrderedEvents(hui.isUsePartialOrderedEvents());
			paramObj.setContraintCosts(hui.getMapConstraintToCost());
			paramObj.setAllowViolatingConstraints(hui.isAllowViolatingConstraints());

			return paramObj;
		} else {
			return null;
		}
	}

	public JComponent constructUI() {
		return new MPReplayParametersUI(transCol, evClassCol, constraints, transCol.size());
	}
}
