package org.processmining.mixedparadigm.parameters;

import info.clearthought.layout.TableLayoutConstants;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.table.DefaultTableModel;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.framework.util.ui.widgets.ProMTable;
import org.processmining.mixedparadigm.models.mixedparadigm.DeclarativeArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayer.algorithms.costbasedcomplete.CostBasedCompleteUI;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class MPReplayParametersUI extends CostBasedCompleteUI {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1750372583381967369L;
	private static final Integer DEFCOSTCONSTRAINT = 2;

	private ProMTable promConstraintTable;
	private DefaultTableModel constraintModel;
	private Map<DeclarativeArc, Integer> mapConstraint2RowIndex = new HashMap<DeclarativeArc, Integer>();;
	private JCheckBox cbAllowViolations;

	public MPReplayParametersUI(Collection<Transition> transCol, Collection<XEventClass> evClassCol,
			Collection<DeclarativeArc> constraints, int modelSize) {
		this(transCol, evClassCol, constraints, null, null, null, null, modelSize);
	}

	public MPReplayParametersUI(Collection<Transition> transCol, Collection<XEventClass> evClassCol,
			Collection<DeclarativeArc> constraints, Map<Transition, Integer> defMoveModelCost,
			Map<Transition, Integer> defSyncCost, Map<XEventClass, Integer> defMoveLogCost,
			Map<DeclarativeArc, Integer> defConstraintCost, int modelSize) {
		super(new double[][] {
				{ TableLayoutConstants.FILL },
				{ 80, 40, 40, TableLayoutConstants.FILL, 35, TableLayoutConstants.FILL, 35, TableLayoutConstants.FILL,
						35, TableLayoutConstants.FILL, 35, 30 } });

		SlickerFactory slickerFactoryInstance = SlickerFactory.instance();

		// row 0
		setTitle(
				slickerFactoryInstance,
				"<html><h1>Set parameters</h1><p>Double click costs on table to change their values. Use only non-negative integers.</p></html>");

		// row 1
		setUpStateSlider(slickerFactoryInstance, 1);

		// row 2
		setupAllowConstraintsCheckbox(slickerFactoryInstance, 2);

		// row 3-8
		setupUI(transCol, evClassCol, defMoveModelCost, defSyncCost, defMoveLogCost, slickerFactoryInstance, 3);

		// row 9-10
		populateConstrViolationPanel(constraints, defConstraintCost, 9);
		final JButton button = populateSetAllButton("0", constraintModel, "0, 10, c, t");

		cbAllowViolations.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				button.setEnabled(cbAllowViolations.isSelected());
			}
		});

		//row 11
		setupPOCheckbox(slickerFactoryInstance, 11);

	}

	protected void setupAllowConstraintsCheckbox(SlickerFactory slickerFactoryInstance, int row) {
		cbAllowViolations = slickerFactoryInstance.createCheckBox("Allow violations of contraints", true);

		add(cbAllowViolations, "0, " + row + ", l, t");

	}

	private void populateConstrViolationPanel(Collection<DeclarativeArc> constraints,
			Map<DeclarativeArc, Integer> defaultCost, int row) {
		// create table to map move on model cost
		List<DeclarativeArc> sortedConstraints = new ArrayList<DeclarativeArc>(constraints);
		Collections.sort(sortedConstraints, new Comparator<DeclarativeArc>() {
			public int compare(DeclarativeArc t1, DeclarativeArc t2) {
				return t1.toString().compareTo(t2.toString());
			}
		});
		Object[][] tableContent = new Object[constraints.size()][2];
		int rowCounter = 0;
		for (DeclarativeArc constraint : sortedConstraints) {
			if (defaultCost != null) {
				tableContent[rowCounter] = new Object[] { constraint.toString(),
						defaultCost.get(constraint) == null ? DEFCOSTCONSTRAINT : defaultCost.get(constraint) };
			} else {
				tableContent[rowCounter] = new Object[] { constraint.toString(), DEFCOSTCONSTRAINT };
			}
			mapConstraint2RowIndex.put(constraint, rowCounter);
			rowCounter++;
		}
		constraintModel = new DefaultTableModel(tableContent, new Object[] { "Constraint", "Cost of violation" }) {
			private static final long serialVersionUID = -6019224467802441949L;

			@Override
			public boolean isCellEditable(int row, int column) {
				return (column != 0) && cbAllowViolations.isSelected();
			};
		};
		promConstraintTable = new ProMTable(constraintModel);
		promConstraintTable.setPreferredSize(new Dimension(700, 200));
		promConstraintTable.setMinimumSize(new Dimension(700, 200));
		add(promConstraintTable, "0, " + row + ", c, t");
	}

	/**
	 * Get map from event class to cost of move on log
	 * 
	 * @return
	 */
	public Map<DeclarativeArc, Integer> getMapConstraintToCost() {
		Map<DeclarativeArc, Integer> mapEvClass2Cost = new HashMap<DeclarativeArc, Integer>();
		for (DeclarativeArc evClass : mapConstraint2RowIndex.keySet()) {
			int index = mapConstraint2RowIndex.get(evClass);
			if (constraintModel.getValueAt(index, 1) instanceof Integer) {
				mapEvClass2Cost.put(evClass, (Integer) constraintModel.getValueAt(index, 1));
			} else {
				try {
					mapEvClass2Cost.put(evClass,
							Integer.parseInt(constraintModel.getValueAt(index, 1).toString().trim()));
				} catch (Exception exc) {
					mapEvClass2Cost.put(evClass, DEFCOSTCONSTRAINT);
				}

			}
		}
		return mapEvClass2Cost;
	}

	public boolean isAllowViolatingConstraints() {
		return cbAllowViolations.isSelected();
	}

}
