package org.processmining.mixedparadigm.plugins;

import java.sql.SQLException;

import org.deckfour.uitopia.api.event.TaskListener;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.mixedparadigm.controller.DatabaseDeclareController;
import org.processmining.mixedparadigm.dialogs.ConfigurationDialogImpl;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.mixedparadigm.parameters.DatabaseDeclareParameters;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.widgets.wizard.Wizard;
import org.processmining.widgets.wizard.WizardResult;

@Plugin(name = "Database-Incremental Declare Miner (DIDM)", parameterLabels = { "Parameters" }, returnLabels = {
		"DebugPetriNet", "Marking" }, userAccessible = true, returnTypes = { PetrinetWithAutomata.class, Marking.class})
public class DIDM {

	@UITopiaVariant(affiliation = "Eindhoven University of Technology", author = "A. Syamsiyah", email = "a.syamsiyah@tue.nl")
	@PluginVariant(variantLabel = "Database Declare", requiredParameterLabels = {})
	public Object[] apply(UIPluginContext context) throws SQLException {

		WizardResult<DatabaseDeclareParameters> wizRes = Wizard.show(context, new ConfigurationDialogImpl(
				context, "Configure Database Connection", new DatabaseDeclareParameters(), null));
		if (wizRes.getInteractionResult().equals(TaskListener.InteractionResult.FINISHED)) {
			return apply(context, wizRes.getParameters());
		} else if (wizRes.getInteractionResult().equals(TaskListener.InteractionResult.CANCEL)) {
			context.getFutureResult(0).cancel(true);
		}
		return null;
	}

	@PluginVariant(variantLabel = "Database-Incremental Declare Miner (DIDM)", requiredParameterLabels = { 0 })
	public Object[] apply(UIPluginContext context,
		DatabaseDeclareParameters parameter) throws SQLException {
		
		// example parameters
		/*parameter.setLogId("1");
		parameter.setDatabaseName("log_7_f_c");
		parameter.setPassword("root");
		parameter.setServer("localhost");
		parameter.setUsername("root");
		parameter.setThreshold(1.0);*/
		
		/*parameter.setLogId("1");
		parameter.setDatabaseName("xerox_declare_1000");
		parameter.setPassword("ProM plug-in");
		parameter.setServer("localhost:33006");
		parameter.setUsername("xerox");
		parameter.setThreshold(1.0);*/
		
		DatabaseDeclareController controller = new DatabaseDeclareController(parameter);
		
		Object[] obj = controller.printProcessModel(context);	

		return obj;
	}
}
