package org.processmining.mixedparadigm.plugins;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.mixedparadigm.models.mixedparadigm.DeclarativeArc;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomataImpl;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.ConstraintType;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.AlternatePrecedence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.AlternateResponse;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.AlternateSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.ChainPrecedence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.ChainResponse;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.ChainSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.CoExistence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.NotChainSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.NotCoExistence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.NotSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.Precedence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.RespondedExistence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.Response;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.Succession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.branched.BranchedPrecedence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.branched.Choice;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.branched.ExclusiveChoice;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.unary.AbsenceN;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.unary.ExactlyN;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.unary.ExistenceN;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.unary.Init;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;

public class HardCodedExamples {

	@Plugin(name = "_Book MP model", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "B.F. van Dongen", email = "b.f.v.dongen@tue.nl", pack = "MixedParadigm")
	public static Object[] debugBookPlugin(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Running example");
		Marking m = new Marking();

		Place p0 = petrinet.addPlace("p0");
		Place p1 = petrinet.addPlace("p1");
		Place p2 = petrinet.addPlace("p2");
		Place p3 = petrinet.addPlace("p3");
		Place p4 = petrinet.addPlace("p4");
		Place p5 = petrinet.addPlace("p5");
		Place p6 = petrinet.addPlace("p6");
		
		Transition as = petrinet.addTransition("Application submitted (As)");
		Transition aa = petrinet.addTransition("Accept application (Aa)");
		Transition da1 = petrinet.addTransition("Decline application (Da)");
		Transition fa = petrinet.addTransition("Finalize application (Fa)");
		Transition sso = petrinet.addTransition("Select and send offer (Sso)");
		Transition ro = petrinet.addTransition("Receive offer (Ro)");
		Transition do1 = petrinet.addTransition("Decline offer (Do)");
		Transition da2 = petrinet.addTransition("Decline application (Da)");
		Transition ao = petrinet.addTransition("Accept offer (Ao)");
		Transition aaa = petrinet.addTransition("Approve and activate application (Aaa)");

		petrinet.addArc(p0, as);
		petrinet.addArc(as, p1);
		petrinet.addArc(p1, da1);
		petrinet.addArc(da1, p5);
		petrinet.addArc(p1, aa);
		petrinet.addArc(aa, p2);
		petrinet.addArc(p2, fa);
		petrinet.addArc(fa, p3);
		petrinet.addArc(p3, da2);
		petrinet.addArc(da2, p5);
		petrinet.addArc(p3, aaa);
		petrinet.addArc(aaa, p5);

		petrinet.addArc(sso, p4);
		petrinet.addArc(p4, ro);
		petrinet.addArc(ro, p6);
		petrinet.addArc(p6, ao);
		petrinet.addArc(p6, do1);
		
		
		petrinet.addConstraint(aa, sso, new Precedence());
		petrinet.addConstraint(da2, ao, new NotCoExistence());
		petrinet.addConstraint(ao, new AbsenceN((short) 2));
		petrinet.addConstraint(ao, aaa, new Succession());
		

		m.add(p0);

		context.addConnection(new InitialMarkingConnection(petrinet, m));

		return new Object[] { petrinet, m };
	}

	
	@Plugin(name = "_Make Debug PetrinetWithAutomata 1", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "B.F. van Dongen", email = "b.f.v.dongen@tue.nl", pack = "MixedParadigm")
	public static Object[] debugPlugin(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		Marking m = new Marking();

		Place p0 = petrinet.addPlace("p0");
		Place p1 = petrinet.addPlace("p1");
		Place p2 = petrinet.addPlace("p2");
		Place p3 = petrinet.addPlace("p3");
		Place p4 = petrinet.addPlace("p4");
		Place p5 = petrinet.addPlace("p5");

		Transition ta = petrinet.addTransition("C");
		Transition tb = petrinet.addTransition("A");
		Transition tc = petrinet.addTransition("B");
		Transition td = petrinet.addTransition("D");
		Transition te = petrinet.addTransition("E");
		Transition tf = petrinet.addTransition("F");

		petrinet.addArc(p0, ta);
		petrinet.addArc(ta, p1);
		petrinet.addArc(ta, p2);

		petrinet.addArc(p1, tb);
		petrinet.addArc(tb, p3);

		petrinet.addArc(p3, tf);
		petrinet.addArc(tf, p1);

		petrinet.addArc(p2, tc);
		petrinet.addArc(tc, p4);

		petrinet.addArc(p3, td);
		petrinet.addArc(p4, td);
		petrinet.addArc(td, p5);

		petrinet.addConstraint(te, te, ConstraintType.Helper.fromName(ConstraintType.INIT));
		petrinet.addConstraint(tc, tf, new Response());
		petrinet.addConstraint(tc, tf, new ExactlyN((short) 2));

		m.add(p0);

		context.addConnection(new InitialMarkingConnection(petrinet, m));

		return new Object[] { petrinet, m };
	}

	@Plugin(name = "_Debug PetrinetWithAutomata PhD Example", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Johannes De Smedt", email = "jdesmed@ed.ac.uk", pack = "MixedParadigm")
	public static Object[] debugPluginMultipleConstraintsExample(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		Marking m = new Marking();

		Transition t1 = petrinet.addTransition("ContentCreation");
		Transition t2 = petrinet.addTransition("JournalPaper");
		Transition t3 = petrinet.addTransition("Conference");
		Transition t4 = petrinet.addTransition("FirstSeminar");
		Transition t5 = petrinet.addTransition("SecondSeminar");
		Transition t6 = petrinet.addTransition("Defense");
		
		Place p1 = petrinet.addPlace("source");
		Place p2 = petrinet.addPlace("p2");
		Place p3 = petrinet.addPlace("p3");
		Place p4 = petrinet.addPlace("sink");
		
		petrinet.addArc(p1, t4);
		petrinet.addArc(t4, p2);
		petrinet.addArc(p2, t5);
		petrinet.addArc(t5, p3);
		petrinet.addArc(p3, t6);
		petrinet.addArc(t6, p4);

		petrinet.addConstraint(t1, new Init());
		petrinet.addConstraint(t1, new ExistenceN((short) 1));
		petrinet.addConstraint(t2, new ExistenceN((short) 1));
		petrinet.addConstraint(t3, new ExistenceN((short) 1));
		petrinet.addConstraint(t4, new AbsenceN((short) 2));
		petrinet.addConstraint(t5, new AbsenceN((short) 2));
		petrinet.addConstraint(t6, new AbsenceN((short) 2));
		
		petrinet.addConstraint(t1, t2, new AlternatePrecedence());
		petrinet.addConstraint(t1, t3, new AlternatePrecedence());
		petrinet.addConstraint(t1, t6, new Succession());
		petrinet.addConstraint(t2, t4, new Precedence());
		petrinet.addConstraint(t2, t6, new Succession());
		petrinet.addConstraint(t3, t5, new Precedence());
		petrinet.addConstraint(t3, t6, new Succession());
		petrinet.addConstraint(t6, t1, new NotSuccession());
		petrinet.addConstraint(t6, t2, new NotSuccession());
		petrinet.addConstraint(t6, t3, new NotSuccession());

		m.add(p1);
		context.addConnection(new InitialMarkingConnection(petrinet, m));

		return new Object[] { petrinet, m };
	}
	
	@Plugin(name = "_Debug PetrinetWithAutomata Mail Example", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Johannes De Smedt", email = "jdesmed@ed.ac.uk", pack = "MixedParadigm")
	public static Object[] debugPluginConstraintsMailExample(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		Marking m = new Marking();

		Transition t1 = petrinet.addTransition("A");
		Transition t2 = petrinet.addTransition("B");
		Transition t4 = petrinet.addTransition("D");
		
		Transition tI = petrinet.addTransition("IN");
		tI.setInvisible(true);
		
		Place p1 = petrinet.addPlace("P1");
		Place p2 = petrinet.addPlace("P2");
		Place p3 = petrinet.addPlace("P3");

//		petrinet.addConstraint(t4, t1, new NotChainSuccession());
//		petrinet.addConstraint(t4, t2, new NotChainSuccession());
		petrinet.addConstraint(t4, t1, new Response());
		petrinet.addConstraint(t4, t2, new Response());
		
		petrinet.addArc(p1, t1);
		petrinet.addArc(t1, p2);
		petrinet.addArc(p2, t2);
		petrinet.addArc(t2, p3);
		petrinet.addArc(p3, tI);
		petrinet.addArc(tI, p2);
		
		context.addConnection(new InitialMarkingConnection(petrinet, m));
		m.add(p1);
		
		Marking mf = new Marking();
		mf.add(p2);
		context.addConnection(new FinalMarkingConnection(petrinet, mf));
		
		return new Object[] { petrinet, m };
	}
	
	@Plugin(name = "_Debug PetrinetWithAutomata Test Example", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Johannes De Smedt", email = "jdesmed@ed.ac.uk", pack = "MixedParadigm")
	public static Object[] debugPluginConstraintsTestExample(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		Marking m = new Marking();

		Transition t1 = petrinet.addTransition("A");
		Transition t2 = petrinet.addTransition("B");
		Transition t3 = petrinet.addTransition("C");
		
		//Transition tI = petrinet.addTransition("tI");
		//tI.setInvisible(true);
		
		Place p1 = petrinet.addPlace("P1");
		Place p2 = petrinet.addPlace("P2");
		//Place p3 = petrinet.addPlace("P3");


		petrinet.addConstraint(t1, t3, new AlternatePrecedence());
		petrinet.addConstraint(t2, t3, new AlternatePrecedence());
		
		petrinet.addArc(p1, t1);
		petrinet.addArc(t1, p2);
		petrinet.addArc(p2, t2);
		petrinet.addArc(t2, p1);
		//petrinet.addArc(t2, p3);
		//petrinet.addArc(p3, tI);
		//petrinet.addArc(tI, p1);
		
		context.addConnection(new InitialMarkingConnection(petrinet, m));
		m.add(p1);
		
		Marking mf = new Marking();
		mf.add(p1);
		context.addConnection(new FinalMarkingConnection(petrinet, mf));
		
		return new Object[] { petrinet, m };
	}
	
	@Plugin(name = "_Debug PetrinetWithAutomata Test2 Example", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Johannes De Smedt", email = "jdesmed@ed.ac.uk", pack = "MixedParadigm")
	public static Object[] debugPluginConstraintsTest2Example(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		Marking m = new Marking();

		Transition t1 = petrinet.addTransition("A");
		Transition t2 = petrinet.addTransition("B");
		Transition t3 = petrinet.addTransition("C");
		Transition t4 = petrinet.addTransition("D");
		Transition t5 = petrinet.addTransition("E");
		
		Transition tI = petrinet.addTransition("tI");
		tI.setInvisible(true);
		
		Place p1 = petrinet.addPlace("P1");
		Place p2 = petrinet.addPlace("P2");
		Place p3 = petrinet.addPlace("P3");
		Place p4 = petrinet.addPlace("P4");
		Place p5 = petrinet.addPlace("P5");
		Place p6 = petrinet.addPlace("P6");

		petrinet.addConstraint(t5, new Init());
		petrinet.addConstraint(t4, t5, new AlternateResponse());
		petrinet.addConstraint(t2, t3, new ChainResponse());
		
		petrinet.addArc(p1, t1);
		petrinet.addArc(t1, p2);
		petrinet.addArc(t1, p3);
		petrinet.addArc(p2, t2);
		petrinet.addArc(p3, t3);
		petrinet.addArc(t2, p4);
		petrinet.addArc(t3, p5);
		petrinet.addArc(p4, t4);
		petrinet.addArc(p5, t4);
		petrinet.addArc(t4, p6);
		petrinet.addArc(p6, tI);
		petrinet.addArc(tI, p1);
		
		context.addConnection(new InitialMarkingConnection(petrinet, m));
		m.add(p1);
		
		Marking mf = new Marking();
		mf.add(p6);
		context.addConnection(new FinalMarkingConnection(petrinet, mf));
		
		return new Object[] { petrinet, m };
	}
	
	@Plugin(name = "_Make Debug PetrinetWithAutomata Double Response", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "B.F. van Dongen", email = "b.f.v.dongen@tue.nl", pack = "MixedParadigm")
	public static Object[] debugPluginDoubleResponse(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		Marking m = new Marking();

		Transition ta = petrinet.addTransition("A");
		Transition tb = petrinet.addTransition("B");

		petrinet.addConstraint(ta, tb, new Response());
		petrinet.addConstraint(tb, new ExistenceN((short) 2));

		context.addConnection(new InitialMarkingConnection(petrinet, m));

		return new Object[] { petrinet, m };
	}

	@Plugin(name = "_Debug PetrinetWithAutomata All Constraints", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "B.F. van Dongen", email = "b.f.v.dongen@tue.nl", pack = "MixedParadigm")
	public static Object[] debugPluginAllConstraints(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		Marking m = new Marking();

		Transition ta1 = petrinet.addTransition("A");
		Transition tb1 = petrinet.addTransition("B");
		Transition tc1 = petrinet.addTransition("C");
		Transition td1 = petrinet.addTransition("D");

		DeclarativeArc arc = petrinet.addConstraint(ta1, tb1, new Choice((short) 2));
		arc.addAdditionalSource(tc1);
		arc.addAdditionalTarget(td1);

		Transition ta2 = petrinet.addTransition("A");
		Transition tb2 = petrinet.addTransition("B");
		Transition tc2 = petrinet.addTransition("C");
		Transition td2 = petrinet.addTransition("D");
		Transition te2 = petrinet.addTransition("E");
		Transition tf2 = petrinet.addTransition("F");

		// BINARY CONSTRAINTS
		petrinet.addConstraint(ta2, tb2, new RespondedExistence());
		petrinet.addConstraint(tb2, tc2, new CoExistence());
		petrinet.addConstraint(tc2, td2, new Response());
		petrinet.addConstraint(td2, te2, new Precedence());
		petrinet.addConstraint(te2, tf2, new Succession());

		Transition ta3 = petrinet.addTransition("A");
		Transition tb3 = petrinet.addTransition("B");
		Transition tc3 = petrinet.addTransition("C");
		Transition td3 = petrinet.addTransition("D");
		Transition te3 = petrinet.addTransition("E");
		Transition tf3 = petrinet.addTransition("F");
		Transition tg3 = petrinet.addTransition("G");

		petrinet.addConstraint(ta3, tb3, new AlternateResponse());
		petrinet.addConstraint(tb3, tc3, new AlternatePrecedence());
		petrinet.addConstraint(tc3, td3, new AlternateSuccession());
		petrinet.addConstraint(td3, te3, new ChainResponse());
		petrinet.addConstraint(te3, tf3, new ChainPrecedence());
		petrinet.addConstraint(tf3, tg3, new ChainSuccession());

		Transition ta4 = petrinet.addTransition("A");
		Transition tb4 = petrinet.addTransition("B");
		Transition tc4 = petrinet.addTransition("C");
		Transition td4 = petrinet.addTransition("D");

		petrinet.addConstraint(ta4, tb4, new NotCoExistence());
		petrinet.addConstraint(tb4, tc4, new NotSuccession());
		petrinet.addConstraint(tc4, td4, new NotChainSuccession());

		// UNARY CONSTRAINTS
		petrinet.addConstraint(ta2, new Init());
		petrinet.addConstraint(ta3, new AbsenceN((short) 4));
		petrinet.addConstraint(ta4, new ExistenceN((short) 4));
		petrinet.addConstraint(tb2, new ExactlyN((short) 2));

		context.addConnection(new InitialMarkingConnection(petrinet, m));

		return new Object[] { petrinet, m };
	}

	@Plugin(name = "_Debug PetrinetWithAutomata Choice Constraint", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "B.F. van Dongen", email = "b.f.v.dongen@tue.nl", pack = "MixedParadigm")
	public static Object[] debugPluginChoiceConstraints(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		Marking m = new Marking();

		Transition ta1 = petrinet.addTransition("A");
		Transition tb1 = petrinet.addTransition("B");
		Transition tc1 = petrinet.addTransition("C");
		Transition td1 = petrinet.addTransition("D");

		DeclarativeArc arc = petrinet.addConstraint(ta1, tb1, new ExclusiveChoice((short) 2));
		arc.addAdditionalSource(tc1);
		arc.addAdditionalTarget(td1);

		context.addConnection(new InitialMarkingConnection(petrinet, m));

		return new Object[] { petrinet, m };
	}

	@Plugin(name = "_Debug PetrinetWithAutomata Cover Example", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "B.F. van Dongen", email = "b.f.v.dongen@tue.nl", pack = "MixedParadigm")
	public static Object[] debugPluginCoverExample(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		Marking m = new Marking();

		Transition t1 = petrinet.addTransition("check X-ray risk");
		Transition t2 = petrinet.addTransition("perform X-ray");
		Transition t3 = petrinet.addTransition("examine patient");
		Transition t4 = petrinet.addTransition("perform reposition");
		Transition t5 = petrinet.addTransition("perform surgery");
		Transition t6 = petrinet.addTransition("prescribe sling");
		Transition t7 = petrinet.addTransition("apply cast");
		Transition t8 = petrinet.addTransition("prescribe rehabilitation");
		Transition t9 = petrinet.addTransition("remove cast");
		Transition t10 = petrinet.addTransition("prescribe medication");

		petrinet.addConstraint(t3, new Init());
		petrinet.addConstraint(t1, t2, new AlternatePrecedence());

		DeclarativeArc arc = petrinet.addConstraint(t4, t5, new Choice((short) 1));
		arc.addAdditionalSource(t6);
		arc.addAdditionalTarget(t7);

		arc = petrinet.addConstraint(t2, t4, new BranchedPrecedence());
		arc.addAdditionalTarget(t5);
		arc.addAdditionalTarget(t7);

		petrinet.addConstraint(t5, t8, new Response());

		Place p = petrinet.addPlace("p0");
		petrinet.addArc(t7, p);
		petrinet.addArc(p, t9);

		context.addConnection(new InitialMarkingConnection(petrinet, m));

		return new Object[] { petrinet, m };
	}

	@Plugin(name = "_Debug PetrinetWithAutomata Init Example", level = PluginLevel.NightlyBuild, categories = { PluginCategory.ConformanceChecking }, returnLabels = {
			"DebugPetriNet", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class }, parameterLabels = {}, help = "Debug Petrinet with automata", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "B.F. van Dongen", email = "b.f.v.dongen@tue.nl", pack = "MixedParadigm")
	public static Object[] debugPluginInitExample(PluginContext context) {
		PetrinetWithAutomata petrinet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		Marking m = new Marking();

		Transition t1 = petrinet.addTransition("A");
		Transition t2 = petrinet.addTransition("B");

		petrinet.addConstraint(t1, new Init());

		context.addConnection(new InitialMarkingConnection(petrinet, m));

		return new Object[] { petrinet, m };
	}

}
