package org.processmining.mixedparadigm.plugins;

import nl.tue.astar.AStarException;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.mixedparadigm.algorithms.replayer.MPReplayResult;
import org.processmining.mixedparadigm.connections.MPReplayResultConnection;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayer.PNLogReplayer;
import org.processmining.plugins.petrinet.replayer.algorithms.IPNReplayAlgorithm;
import org.processmining.plugins.petrinet.replayer.algorithms.IPNReplayParameter;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;

@Plugin(name = "Replay a Log on Mixed Paradigm model for Conformance Analysis", level = PluginLevel.NightlyBuild, //
categories = { PluginCategory.ConformanceChecking }, returnLabels = { "Petrinet log replay result" }, //
returnTypes = { MPReplayResult.class }, parameterLabels = { "Petri net", "Event Log", "Mapping", "Replay Algorithm",
		"Parameters" }, //
help = "Replay an event log on a mixed paradigm model to check conformance.", userAccessible = true)
public class MPReplayer {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "B.F. van Dongen", email = "b.f.v.dongen@tue.nl", pack = "MixedParadigm")
	@PluginVariant(variantLabel = "From Mixed Paradigm net and Event Log", requiredParameterLabels = { 0, 1 })
	public MPReplayResult replayLog(final UIPluginContext context, PetrinetWithAutomata net, XLog log)
			throws ConnectionCannotBeObtained, AStarException {

		final MPReplayResult mpResult = new MPReplayResult();
		PNRepResult result = (new PNLogReplayer() {

			// Create new connection to MPReplayResult rather than PNRepResult
			@Override
			protected void createConnections(PluginContext context, PetrinetGraph net, XLog log,
					TransEvClassMapping mapping, IPNReplayAlgorithm selectedAlg, IPNReplayParameter parameters,
					PNRepResult replayRes) {
				// override the connection and set it to the appropriate type.
				context.addConnection(new MPReplayResultConnection("Connection between replay result, "
						+ XConceptExtension.instance().extractName(log) + ", and " + net.getLabel(),
						(PetrinetWithAutomata) net, log, mapping, selectedAlg, parameters, mpResult));
			}
		}).replayLogGUI(context, net, log);

		mpResult.setPNRepResult(result);

		return mpResult;
	}
}
