package org.processmining.mixedparadigm.plugins;

import javax.swing.JComponent;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.mixedparadigm.algorithms.replayer.MPReplayResult;
import org.processmining.mixedparadigm.connections.MPReplayResultConnection;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.mixedparadigm.visualisation.MPReplayProjectedOnModelPanel;
import org.processmining.models.connections.petrinets.PNRepResultAllRequiredParamConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;

/**
 * @author aadrians Oct 26, 2011
 * 
 */
@Plugin(name = "Model Projected with Alignments", level = PluginLevel.NightlyBuild,//
returnLabels = { "Mixed Paradigm Log-Model Alignment Projected on Model" }, //
returnTypes = { JComponent.class }, parameterLabels = { "Mixed Paradigm Log-Model alignment" },//
userAccessible = true)
@Visualizer
public class MPResultProjectedOnModelVisualizer {
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, MPReplayResult logReplayResult)
			throws ConnectionCannotBeObtained {
		//		System.gc();
		PetrinetWithAutomata net = null;
		XLog log = null;
		TransEvClassMapping map = null;
		Marking initMarking = null;
		try {
			MPReplayResultConnection conn = context.getConnectionManager().getFirstConnection(
					MPReplayResultConnection.class, context, logReplayResult);

			net = conn.getObjectWithRole(PNRepResultAllRequiredParamConnection.PN);
			log = conn.getObjectWithRole(PNRepResultAllRequiredParamConnection.LOG);
			map = conn.getObjectWithRole(PNRepResultAllRequiredParamConnection.TRANS2EVCLASSMAPPING);

		} catch (Exception exc) {
			context.log("No mixed paradigm net can be found for this log replay result");
			context.getFutureResult(0).cancel(true);
			return null;
		}

		try {
			// get initial marking of the net
			InitialMarkingConnection initMarkingConn = context.getConnectionManager().getFirstConnection(
					InitialMarkingConnection.class, context, net);
			initMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);
		} catch (Exception exc) {
			// it's okay not to have any initial marking
			initMarking = new Marking();
		}
		return new MPReplayProjectedOnModelPanel(context, net, initMarking, log, map, logReplayResult);
	}
}
