package org.processmining.mixedparadigm.plugins;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.mixedparadigm.algorithms.replayer.MPReplayResult;
import org.processmining.mixedparadigm.connections.MPReplayResultConnection;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.models.connections.petrinets.PNRepResultAllRequiredParamConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.pnalignanalysis.visualization.performancematrix.PerformanceMatrixCreator;

/**
 * @author aadrians Jul 19, 2012
 * 
 */
@Plugin(name = "Time Between Transition Analysis", level = PluginLevel.NightlyBuild, //
returnLabels = { "Performance Matrix" }, returnTypes = { JComponent.class }, //
parameterLabels = { "Mixed Paradigm Log-Model alignment" }, userAccessible = true)
@Visualizer
public class MPResultTimeBetweenTransitionVisualizer {
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, MPReplayResult logReplayResult) {
		PetrinetWithAutomata net = null;
		XLog log = null;
		Marking initMarking = null;
		try {
			MPReplayResultConnection conn = context.getConnectionManager().getFirstConnection(
					MPReplayResultConnection.class, context, logReplayResult);

			net = conn.getObjectWithRole(PNRepResultAllRequiredParamConnection.PN);
			log = conn.getObjectWithRole(PNRepResultAllRequiredParamConnection.LOG);
		} catch (Exception exc) {
			context.log("No net can be found for this log replay result");
			context.getFutureResult(0).cancel(true);
			return null;
		}

		try {
			// get initial marking of the net
			InitialMarkingConnection initMarkingConn = context.getConnectionManager().getFirstConnection(
					InitialMarkingConnection.class, context, net);
			initMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);
		} catch (Exception exc) {
			// it's okay not to have any initial marking
			initMarking = new Marking();
		}

		int option = JOptionPane.showConfirmDialog(new JPanel(), "Show invisible transitions in the matrix?",
				"Performance Matrix Setting", JOptionPane.YES_NO_OPTION);
		return PerformanceMatrixCreator.createPerformanceMatrix(logReplayResult.getPNRepResult(), net, log,
				initMarking, option == 0);
	}

}