package org.processmining.mixedparadigm.plugins;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.jgraph.ProMJGraphVisualizer;
import org.processmining.models.jgraph.visualization.ProMJGraphPanel;
import org.processmining.models.semantics.petrinet.Marking;

@Plugin(name = "@0 Visualize Mixed Paradigm model", level = PluginLevel.PeerReviewed, returnLabels = { "Visualized Mixed Paradigm Model" }, returnTypes = { JComponent.class }, parameterLabels = {
		"Petri net", "Initial Marking" }, userAccessible = true)
@Visualizer
public class MPVisualizer {

	@PluginVariant(requiredParameterLabels = { 0, 1 })
	public JComponent visualize(PluginContext context, PetrinetWithAutomata n, Marking m) {
		return visualizeWithAll(context, n, m);
	}

	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, PetrinetWithAutomata n) {
		return visualizeWithAll(context, n, null);
	}

	protected JComponent visualizeWithAll(PluginContext context, PetrinetGraph n, Marking m) {

		ConnectionManager cm = context.getConnectionManager();
		// Try to see if there is an initial marking available for visualization
		try {
			for (InitialMarkingConnection mnc : cm.getConnections(InitialMarkingConnection.class, context, n)) {
				m = mnc.getObjectWithRole(InitialMarkingConnection.MARKING);

			}
		} catch (ConnectionCannotBeObtained e) {
			// no initial marking, no problem
			m = new Marking();
		}

		//		return PetriNetVisualization.getComponent(context, n, m);

		ViewSpecificAttributeMap map = new ViewSpecificAttributeMap();
		for (Place p : m) {
			String label = "" + m.occurrences(p);
			map.putViewSpecific(p, AttributeMap.LABEL, label);
			map.putViewSpecific(p, AttributeMap.TOOLTIP, p.getLabel());
			map.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
		}

		ProMJGraphPanel panel = ProMJGraphVisualizer.instance().visualizeGraph(context, n, map);

		return panel;
	}
}
