package org.processmining.mixedparadigm.plugins;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;

import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.mixedparadigm.plugins.pnml.DPnmlElementFactory;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.pnml.base.Pnml;
import org.processmining.plugins.pnml.base.Pnml.PnmlType;
import org.processmining.plugins.utils.ProvidedObjectHelper;

@Plugin(name = "DPNML export (Petri net with automata)", level = PluginLevel.NightlyBuild, returnLabels = {}, //
returnTypes = {}, parameterLabels = { "Petrinet with Automata", "File" }, userAccessible = true)
@UIExportPlugin(description = "DPNML files", extension = "dpnml")
public class PnmlExportMPModel {

	@PluginVariant(variantLabel = "DPNML export (Petri net with automata)", requiredParameterLabels = { 0, 1 })
	public void exportPetriNetToPNMLFile(PluginContext context, PetrinetWithAutomata net, File file) throws IOException {
		exportPetriNetToPNMLOrEPNMLFile(context, net, file);
	}

	protected void exportPetriNetToPNMLOrEPNMLFile(PluginContext context, PetrinetWithAutomata net, File file)
			throws IOException {
		Marking marking;
		try {
			marking = context.tryToFindOrConstructFirstObject(Marking.class, InitialMarkingConnection.class,
					InitialMarkingConnection.MARKING, net);
		} catch (ConnectionCannotBeObtained e) {
			// use empty marking\
			marking = new Marking();
		}
		Collection<Marking> finalMarkings = new HashSet<Marking>();
		try {
			Collection<FinalMarkingConnection> connections = context.getConnectionManager().getConnections(
					FinalMarkingConnection.class, context, net);
			for (FinalMarkingConnection connection : connections) {
				finalMarkings.add((Marking) connection.getObjectWithRole(FinalMarkingConnection.MARKING));
			}
		} catch (ConnectionCannotBeObtained e) {
		}

		GraphLayoutConnection layout;
		try {
			layout = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, net);
		} catch (ConnectionCannotBeObtained e) {
			layout = new GraphLayoutConnection(net);
		}
		HashMap<PetrinetGraph, Marking> markedNets = new HashMap<PetrinetGraph, Marking>();
		HashMap<PetrinetGraph, Collection<Marking>> finalMarkedNets = new HashMap<PetrinetGraph, Collection<Marking>>();
		markedNets.put(net, marking);
		finalMarkedNets.put(net, finalMarkings);

		Pnml pnml = new Pnml();
		DPnmlElementFactory factory = new DPnmlElementFactory();
		synchronized (factory) {
			pnml.setFactory(factory);
			pnml = pnml.convertFromNet(markedNets, finalMarkedNets, layout);
		}
		pnml.setType(PnmlType.PNML);
		updateName(context, pnml, net);
		String text = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n" + pnml.exportElement(pnml);

		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));
		bw.write(text);
		bw.close();
	}

	private static void updateName(PluginContext context, Pnml pnml, PetrinetGraph net) {
		String name = ProvidedObjectHelper.getProvidedObjectLabel(context, net);
		if (name != null) {
			pnml.setName(name);
		}
	}

}
