package org.processmining.mixedparadigm.plugins;

import java.io.InputStream;

import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomataImpl;
import org.processmining.mixedparadigm.plugins.pnml.DPnmlElementFactory;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.pnml.base.Pnml;
import org.processmining.plugins.pnml.importing.PnmlImportUtils;

@Plugin(name = "Import Petri net with Automata from DPNML file", level = PluginLevel.NightlyBuild, parameterLabels = { "Filename" }, returnLabels = {
		"Petri Net", "Marking" }, returnTypes = { PetrinetWithAutomata.class, Marking.class })
@UIImportPlugin(description = "DPNML Petri net files", extensions = { "dpnml" })
public class PnmlImportMPModel extends AbstractImportPlugin {

	protected FileFilter getFileFilter() {
		return new FileNameExtensionFilter("DPNML files", "dpnml");
	}

	protected Object importFromStream(PluginContext context, InputStream input, String filename, long fileSizeInBytes)
			throws Exception {
		PnmlImportUtils utils = new PnmlImportUtils();
		DPnmlElementFactory factory = new DPnmlElementFactory();
		Pnml pnml = utils.importPnmlFromStream(context, factory, input, filename, fileSizeInBytes);
		if (pnml == null) {
			/*
			 * No PNML found in file. Fail.
			 */
			return null;
		}
		/*
		 * PNML file has been imported. Now we need to convert the contents to a
		 * regular Petri net.
		 */
		PetrinetGraph net = new PetrinetWithAutomataImpl(pnml.getLabel());

		return utils.connectNet(context, pnml, net);
	}
}
