package org.processmining.mixedparadigm.plugins.pnml;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.mixedparadigm.models.mixedparadigm.DeclarativeArc;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.AbstractGraphElement;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.ExpandableSubNet;
import org.processmining.models.graphbased.directed.petrinet.elements.InhibitorArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.ResetArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.pnml.base.Pnml;
import org.processmining.plugins.pnml.elements.PnmlArc;
import org.processmining.plugins.pnml.elements.PnmlNode;
import org.processmining.plugins.pnml.elements.PnmlPage;
import org.xmlpull.v1.XmlPullParser;

public class DPnmlPage extends PnmlPage {

	protected List<PnmlDeclarativeArc> declarativeArcList;

	protected DPnmlPage() {
		super();
		declarativeArcList = new ArrayList<PnmlDeclarativeArc>();
	}

	@Override
	protected void convertArcsToNet(PetrinetGraph net, ExpandableSubNet subNet, Marking marking,
			Map<String, Place> placeMap, Map<String, Transition> transitionMap,
			Map<String, PetrinetEdge<?, ?>> edgeMap, Point2D.Double displacement, GraphLayoutConnection layout) {
		for (PnmlArc arc : arcList) {
			arc.convertToNet(net, this.subNet, placeMap, transitionMap, edgeMap, displacement, layout);
		}

		for (PnmlDeclarativeArc arc : declarativeArcList) {
			arc.convertToNet(net, this.subNet, placeMap, transitionMap, edgeMap, displacement, layout);
		}

		for (PnmlNode node : nodeList) {
			if (node instanceof DPnmlPage) {
				((DPnmlPage) node).convertArcsToNet(net, subNet, marking, placeMap, transitionMap, edgeMap,
						getDisplacement(displacement, (PnmlPage) node), layout);
			}
		}
	}

	@Override
	protected void convertArcsFromNet(PetrinetGraph net, Marking marking, ExpandableSubNet parent,
			ExpandableSubNet element, Map<Pair<AbstractGraphElement, ExpandableSubNet>, String> idMap, PageMap pageMap,
			GraphLayoutConnection layout) {
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : net.getEdges()) {
			if (edge instanceof Arc || edge instanceof ResetArc || edge instanceof InhibitorArc) {
				((DPnmlPage) pageMap.get(edge.getParent())).arcList.add(factory.createPnmlArc().convertFromNet(element,
						edge, this, idMap, layout));
			} else if (edge instanceof DeclarativeArc) {
				((DPnmlPage) pageMap.get(edge.getParent())).declarativeArcList
						.add(((DPnmlElementFactory) factory).createPnmlDeclarativeArc().convertFromNet(element,
								(DeclarativeArc) edge, this, idMap, layout));
			}
		}
	}

	@Override
	protected DPnmlPage convertGroupsFromNet(PetrinetGraph net, Marking marking, ExpandableSubNet parent,
			ExpandableSubNet element, Map<Pair<AbstractGraphElement, ExpandableSubNet>, String> idMap, PageMap pageMap,
			GraphLayoutConnection layout) {
		super.convertGroupsFromNet(net, marking, parent, element, idMap, pageMap, layout);
		declarativeArcList = new ArrayList<>();
		return this;
	}

	/**
	 * Checks whether the current start tag is known. If known, it imports the
	 * corresponding child element and returns true. Otherwise, it returns
	 * false.
	 * 
	 * @return Whether the start tag was known.
	 */
	protected boolean importElements(XmlPullParser xpp, Pnml pnml) {
		if (super.importElements(xpp, pnml)) {
			/*
			 * Start tag corresponds to a known child element of a PNML node.
			 */
			return true;
		}
		if (xpp.getName().equals(PnmlDeclarativeArc.TAG)) {
			/*
			 * Found arc start tag. Create an arc object and import arc element.
			 */
			PnmlDeclarativeArc arc = ((DPnmlElementFactory) factory).createPnmlDeclarativeArc();
			arc.importElement(xpp, pnml);
			declarativeArcList.add(arc);
			return true;
		}
		/*
		 * Unknown start tag.
		 */
		return false;
	}

	/**
	 * Exports all child elements.
	 */
	protected String exportElements(Pnml pnml) {
		String s = super.exportElements(pnml, "\n");
		for (PnmlDeclarativeArc arc : declarativeArcList) {
			s += arc.exportElement(pnml);
			s += "\n";
		}

		return s;
	}

}
