package org.processmining.mixedparadigm.plugins.pnml;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.AbstractGraphElement;
import org.processmining.models.graphbased.directed.petrinet.elements.ExpandableSubNet;
import org.processmining.plugins.pnml.base.Pnml;
import org.processmining.plugins.pnml.elements.graphics.PnmlPosition;
import org.xmlpull.v1.XmlPullParser;

public class DPnmlSingleArcGraphics extends DPnmlGraphics {
	/**
	 * PNML annotation graphics tag.
	 */
	public final static String TAG = "singleArcgraphics";

	/**
	 * Positions elements (may be multiple).
	 */
	protected final List<PnmlPosition> positionList;

	/**
	 * Creates a fresh PNML arc graphics.
	 */
	protected DPnmlSingleArcGraphics() {
		this(TAG);
	}

	/**
	 * Creates a fresh PNML arc graphics.
	 */
	protected DPnmlSingleArcGraphics(String tag) {
		super(tag);
		positionList = new ArrayList<PnmlPosition>();
	}

	/**
	 * Checks whether the current start tag is known. If known, it imports the
	 * corresponding child element and returns true. Otherwise, it returns
	 * false.
	 * 
	 * @return Whether the start tag was known.
	 */
	protected boolean importElements(XmlPullParser xpp, Pnml pnml) {
		if (super.importElements(xpp, pnml)) {
			return true;
		}
		if (xpp.getName().equals(PnmlPosition.TAG)) {
			/*
			 * Position element.
			 */
			PnmlPosition position = factory.createPnmlPosition();
			position.importElement(xpp, pnml);
			positionList.add(position);
			return true;
		}
		/*
		 * Unknown start tag.
		 */
		return false;
	}

	/**
	 * Exports the arc graphics.
	 */
	protected String exportElements(Pnml pnml) {
		String s = super.exportElements(pnml);
		for (PnmlPosition position : positionList) {
			s += position.exportElement(pnml);
		}
		return s;
	}

	/**
	 * Sets the graphics for the given graph element.
	 * 
	 * @param subNet
	 *            The given sub net.
	 * @param element
	 *            The given element.
	 * @param displacement
	 *            The displacement for this sub net.
	 */
	public void convertToNet(ExpandableSubNet subNet, AbstractGraphElement element, Point2D.Double displacement,
			GraphLayoutConnection layout) {
		ArrayList<Point2D> points = new ArrayList<Point2D>();
		for (PnmlPosition position : positionList) {
			position.convertToNet(subNet, element, points, displacement);
		}
		layout.setEdgePoints(element, points);
	}

	public DPnmlSingleArcGraphics convertFromNet(ExpandableSubNet parent, AbstractGraphElement element,
			GraphLayoutConnection layout) {
		DPnmlSingleArcGraphics result = null;
		try {
			List<Point2D> points = layout.getEdgePoints(element);
			for (Point2D point : points) {
				positionList.add(factory.createPnmlPosition().convertFromNet(parent, point));
			}
			result = this;
		} catch (Exception ex) {
		}
		return result;
	}

}
