package org.processmining.mixedparadigm.visualisation;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.List;
import java.util.Set;

import org.processmining.mixedparadigm.models.mixedparadigm.DeclarativeArc;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.petrinet.replayresult.visualization.ProcessInstanceConformanceView;

public class MPProcessInstanceConformanceView extends ProcessInstanceConformanceView {

	private static final long serialVersionUID = 7392799293488443117L;
	private List<Set<DeclarativeArc>> violatedConstraints;

	public MPProcessInstanceConformanceView(String traceLabel, List<Object> nodeInstance, List<StepTypes> stepTypes,
			List<Set<DeclarativeArc>> violatedConstraints, int elementWidth) {
		super(traceLabel, nodeInstance, stepTypes, elementWidth);
		this.violatedConstraints = violatedConstraints;
	}

	@Override
	protected void drawEventFlag(Graphics2D g2d, int index, int x, int y, int height) throws IndexOutOfBoundsException {
		if (nodeInstance != null) {
			String names[] = new String[violatedConstraints.get(index).size() + 2];
			names[0] = stepTypes.get(index).toString();
			names[1] = nodeInstance.get(index).toString();
			int i = 2;
			for (DeclarativeArc a : violatedConstraints.get(index)) {
				names[i++] = "Violates: " + a.toString();
			}
			//names[names.length - 1] = "TEST";

			drawMultiLineFlag(g2d, getColorOfIndex(index), x, y, height, names);
		} else {
			super.drawXEventFlag(g2d, index, x, y, height);
		}
	}

	@Override
	protected Color getColorOfIndex(int index) {
		if (violatedConstraints.get(index).isEmpty()) {
			return getColor(stepTypes.get(index));
		} else if (stepTypes.get(index) == StepTypes.LMGOOD) {
			return MPReplayProjectedOnLogPanel.VIOLATINGSYNCHRONOUSCOLOR;
		} else {
			return MPReplayProjectedOnLogPanel.VIOLATINGMOVEMODELCOLOR;
		}
	}

	protected String getStepTypeNameOfIndex(int index) {
		if (violatedConstraints.get(index).isEmpty()) {
			return stepTypes.get(index).toString();
		} else if (stepTypes.get(index) == StepTypes.LMGOOD) {
			return "Violating Synchronous Move";
		} else {
			return "Violating Model Move";
		}
	}

}
