package org.processmining.mixedparadigm.visualisation;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.Component;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.plugin.Progress;
import org.processmining.mixedparadigm.algorithms.replayer.MPReplayResult;
import org.processmining.mixedparadigm.algorithms.replayer.MPSyncReplayResult;
import org.processmining.plugins.petrinet.replayresult.visualization.PNLogReplayResultVisPanel;
import org.processmining.plugins.petrinet.replayresult.visualization.ProcessInstanceConformanceView;
import org.processmining.plugins.petrinet.visualization.AlignmentConstants;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class MPReplayProjectedOnLogPanel extends PNLogReplayResultVisPanel {

	private static final long serialVersionUID = -1333139449755505773L;

	//	public static final Color MOVEMODELREALCOLOR = new Color(205, 106, 205);
	//	public static final Color MOVESYNCCOLOR = Color.GREEN;

	static final Color VIOLATINGSYNCHRONOUSCOLOR = new Color(0, 255, 191);
	static final Color VIOLATINGMOVEMODELCOLOR = new Color(131, 106, 205);

	protected MPReplayResult mpReplayResult;

	public MPReplayProjectedOnLogPanel(XLog log, MPReplayResult mpReplayResult, Progress progress) {
		super(log, mpReplayResult.getPNRepResult(), progress);
		this.mpReplayResult = mpReplayResult;
	}

	@Override
	protected ProcessInstanceConformanceView createProcessInstanceView(SyncReplayResult res, List<Object> result) {
		return new MPProcessInstanceConformanceView("Alignment", result, res.getStepTypes(),
				((MPSyncReplayResult) res).getViolatedConstraints(), 10);
	}

	@Override
	protected Component createLegendPanel() {
		SlickerFactory factory = SlickerFactory.instance();

		JPanel legendPanel = new JPanel();
		legendPanel.setBorder(BorderFactory.createEmptyBorder());
		legendPanel.setBackground(new Color(192, 192, 192));
		TableLayout layout = new TableLayout(new double[][] { { 0.10, TableLayout.FILL }, {} });
		legendPanel.setLayout(layout);

		layout.insertRow(0, 0.2);

		int row = 1;

		layout.insertRow(row, TableLayout.PREFERRED);
		JLabel label = factory.createLabel("LEGEND");
		legendPanel.add(label, "0,1,1,1,c, c");
		row++;

		layout.insertRow(row, 0.2);

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel greenPanel = new JPanel();
		greenPanel.setBackground(AlignmentConstants.MOVESYNCCOLOR);
		legendPanel.add(greenPanel, "0," + row + ",r, c");
		label = factory.createLabel("-Synchronous move (move log+model)");
		legendPanel.add(label, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel greyPanel = new JPanel();
		greyPanel.setBackground(AlignmentConstants.MOVEMODELINVICOLOR);
		legendPanel.add(greyPanel, "0," + row + ",r, c");
		label = factory.createLabel("-Unobservable move (move model only)");
		legendPanel.add(label, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel purplePanel = new JPanel();
		purplePanel.setBackground(AlignmentConstants.MOVEMODELREALCOLOR);
		legendPanel.add(purplePanel, "0," + row + ",r, c");
		label = factory.createLabel("-Skipped event class (move model only)");
		legendPanel.add(label, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel yellowPanel = new JPanel();
		yellowPanel.setBackground(AlignmentConstants.MOVELOGCOLOR);
		legendPanel.add(yellowPanel, "0," + row + ",r, c");
		label = factory.createLabel("-Inserted event class (move log only)");
		legendPanel.add(label, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel orangePanel = new JPanel();
		orangePanel.setBackground(VIOLATINGSYNCHRONOUSCOLOR);
		legendPanel.add(orangePanel, "0," + row + ",r, c");
		label = factory.createLabel("-Synchronous move violating constraints (move log+model)");
		legendPanel.add(label, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel redPanel = new JPanel();
		redPanel.setBackground(VIOLATINGMOVEMODELCOLOR);
		legendPanel.add(redPanel, "0," + row + ",r, c");
		label = factory.createLabel("-Skipped event class violating constraints (move model only)");
		legendPanel.add(label, "1," + row++ + ",l, c");

		return legendPanel;
	}

}
