package org.processmining.mixedparadigm.visualisation;

import java.awt.Color;
import java.util.List;

import org.deckfour.xes.model.XLog;
import org.jgraph.event.GraphSelectionEvent;
import org.jgraph.event.GraphSelectionListener;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.mixedparadigm.algorithms.replayer.MPReplayResult;
import org.processmining.mixedparadigm.models.mixedparadigm.DeclarativeArc;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.jgraph.DirectedGraphElementForMultiEdge;
import org.processmining.models.jgraph.elements.ProMGraphEdge;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.visualization.AlignmentConstants;
import org.processmining.plugins.pnalignanalysis.visualization.projection.CoreInfoProvider;
import org.processmining.plugins.pnalignanalysis.visualization.projection.LogSource;
import org.processmining.plugins.pnalignanalysis.visualization.projection.PNLogReplayProjectedVisPanel;
import org.processmining.plugins.pnalignanalysis.visualization.projection.StatPanel;
import org.processmining.plugins.pnalignanalysis.visualization.projection.TransConfDecorator;

public class MPReplayProjectedOnModelPanel extends PNLogReplayProjectedVisPanel {

	private static final long serialVersionUID = -1549712172159083567L;
	protected MPReplayResult mpReplayResult;

	// Overwrites provider in supertype.
	protected MPCoreInfoProvider provider;

	protected PetrinetWithAutomata net;

	public MPReplayProjectedOnModelPanel(PluginContext context, PetrinetWithAutomata net, Marking initMarking,
			XLog log, TransEvClassMapping map, MPReplayResult logReplayResult) throws ConnectionCannotBeObtained {
		this(context, net, initMarking, log, map, logReplayResult, new LogSource(log));
	}

	public MPReplayProjectedOnModelPanel(PluginContext context, PetrinetWithAutomata net, Marking initMarking,
			XLog log, TransEvClassMapping map, MPReplayResult logReplayResult, LogSource logExporter)
			throws ConnectionCannotBeObtained {
		super(context, net, initMarking, log, map, logReplayResult.getPNRepResult(), logExporter);

		this.mpReplayResult = logReplayResult;
		this.net = (PetrinetWithAutomata) super.net;

		super.initialize(context);

		graph.addGraphSelectionListener(new GraphSelectionListener() {
			public void valueChanged(GraphSelectionEvent e) {
				// selection of a transition would change the stats
				DeclarativeArc arc = null;
				if (e.getCell() instanceof ProMGraphEdge) {
					DirectedGraphEdge<?, ?> cell = ((ProMGraphEdge) e.getCell()).getEdge();
					if (cell instanceof DirectedGraphElementForMultiEdge) {
						arc = (DeclarativeArc) ((DirectedGraphElementForMultiEdge) cell).getMultiEdge();
					} else if (cell instanceof DeclarativeArc) {
						arc = (DeclarativeArc) cell;
					}
				}

				if (arc != null) {
					((MPStatPanel) statPanel).setConstraint(arc);

					boolean[] involvedPlacesFlag = statPanel.getInvolvedPlaces();
					Place[] placeArray = provider.getPlaceArray();
					graph.getModel().beginUpdate();
					for (int i = 0; i < involvedPlacesFlag.length; i++) {
						if (involvedPlacesFlag[i]) {
							graph.getViewSpecificAttributes().putViewSpecific(placeArray[i], AttributeMap.FILLCOLOR,
									involvedMoveOnLogColor);
						} else {
							if (placeWithMoveOnLog[i]) {
								graph.getViewSpecificAttributes().putViewSpecific(placeArray[i],
										AttributeMap.FILLCOLOR, AlignmentConstants.MOVELOGCOLOR);
							} else {
								graph.getViewSpecificAttributes().putViewSpecific(placeArray[i],
										AttributeMap.FILLCOLOR, transparentColor);
							}
						}
					}
					graph.getModel().endUpdate();
					graph.refresh();
				}
			}
		});

	}

	@Override
	protected void initialize(PluginContext context) {
		// Skip
	}

	@Override
	protected CoreInfoProvider createCoreInfoProvider(XLog log, TransEvClassMapping map, PNRepResult logReplayResult) {
		this.provider = new MPCoreInfoProvider(net, marking, map, log, mpReplayResult);
		return this.provider;

	}

	@Override
	protected TransConfDecorator createTransitionDecorator(Transition[] transArr, int pointer, int[] info) {
		return new TransConfDecorator(info[3], info[0], transArr[pointer].getLabel());
	}

	@Override
	protected void setPetrinetEdgeAttributes(ViewSpecificAttributeMap map, float suggestedArcWidth, int intensity,
			PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge) {
		if (!(edge instanceof DeclarativeArc)) {
			super.setPetrinetEdgeAttributes(map, suggestedArcWidth, intensity, edge);
		}
	}

	private void setAttributeValue(ViewSpecificAttributeMap map, DeclarativeArc edge, String key, Object value) {
		// get the layoutConnection
		List<DirectedGraphElement> elements = graph.getLayoutConnection().getMultiGraphElements(edge);
		for (DirectedGraphElement elt : elements) {
			map.putViewSpecific(elt, key, value);
		}
		map.putViewSpecific(edge, key, value);

	}

	@Override
	public void constructVisualization(ViewSpecificAttributeMap map, boolean isShowMoveLogModel, boolean isShowMoveModel) {
		graph.getModel().beginUpdate();

		// Do all the updates to the Petrinet part
		doUpdateInternal(map, isShowMoveLogModel, isShowMoveModel);

		double violBound = provider.getMaximumViolations() - provider.getMinimumViolations();

		// Now for the declarative arcs
		for (DeclarativeArc arc : net.getConstraints()) {
			if (provider.getViolationsFor(arc) > 0) {
				int a = provider.getViolationsFor(arc) - provider.getMinimumViolations();
				int red = 155 + (int) (100.0 * a / violBound);
				setAttributeValue(graph.getViewSpecificAttributes(), arc, AttributeMap.EDGECOLOR, //
						new Color(red, 0, 0));
			}
			float f = getAppropriateStrokeWidth(provider.getActivationsFor(arc));
			f /= Math.sqrt(graph.getViewSpecificAttributes().get(arc, AttributeMap.NUMLINES, 1));
			setAttributeValue(graph.getViewSpecificAttributes(), arc, AttributeMap.LINEWIDTH, f);

		}

		graph.getModel().endUpdate();
		graph.refresh();
		graph.revalidate();
		graph.repaint();
	}

	@Override
	protected StatPanel createStatPanel(CoreInfoProvider provider) {
		return new MPStatPanel((MPCoreInfoProvider) provider);
	}

}
