package org.processmining.mixedparadigm.visualisation;

import java.util.Arrays;

import org.processmining.mixedparadigm.models.mixedparadigm.DeclarativeArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.pnalignanalysis.visualization.projection.CoreInfoProvider;
import org.processmining.plugins.pnalignanalysis.visualization.projection.StatPanel;

public class MPStatPanel extends StatPanel {

	private static final long serialVersionUID = -3474064680949881230L;

	public MPStatPanel(MPCoreInfoProvider infoProvider) {
		super(infoProvider);
	}

	@Override
	protected Object[] createComboItems(CoreInfoProvider infoProvider) {
		Object[] items = new Object[infoProvider.getNumPlaces() + infoProvider.getNumTrans()
				+ ((MPCoreInfoProvider) infoProvider).getNumConstraints() + 1];
		items[0] = "All elements, aggregated";
		int counter = 1;
		for (Transition t : infoProvider.getTransArray()) {
			items[counter++] = t;
		}
		for (Place p : infoProvider.getPlaceArray()) {
			items[counter++] = p.getLabel();
		}
		for (DeclarativeArc arc : ((MPCoreInfoProvider) infoProvider).getConstraintArray()) {
			items[counter++] = arc.toString();
		}

		return items;
	}

	@Override
	protected void updateStats() {
		if (selection.getSelectedIndex() == 0) {
			showAllStats();
		} else {
			// check if its trans or places
			if (selection.getSelectedIndex() <= getInfoProvider().getNumTrans()) {
				showTransStats(selection.getSelectedIndex() - 1);
			} else if (selection.getSelectedIndex() <= getInfoProvider().getNumTrans()
					+ getInfoProvider().getNumPlaces()) {
				showPlaceStats(selection.getSelectedIndex() - 1);
			} else {
				showConstraintStats(selection.getSelectedIndex() - 1);
			}
		}
	}

	protected void showConstraintStats(int selectedIndex) {

		// reset involved place flags
		Arrays.fill(involvedPlaceFlags, false);

		//		int[] infoNumber = getInfoProvider().getInfoNode(selectedIndex);

		Object[][] info; // result
		int arc = selectedIndex - getInfoProvider().getNumTrans() - getInfoProvider().getNumPlaces();
		int[] infoNumber = getInfoProvider().getConstraintInfo(arc);

		info = new Object[6][2];
		info[0] = new Object[] { "#Activations (total)", infoNumber[0] };
		info[1] = new Object[] { "#Activations (in 100% fitting traces)", infoNumber[1] };
		info[2] = new Object[] { "#Traces where Activation occured", infoNumber[2] };
		info[3] = new Object[] { ' ', ' ' };
		info[4] = new Object[] { "#Violations (total)", infoNumber[3] };

		tableModel.setDataVector(info, columnIdentifier);
		table.doLayout();
	}

	@Override
	protected MPCoreInfoProvider getInfoProvider() {
		return (MPCoreInfoProvider) super.getInfoProvider();
	}

	public void setConstraint(DeclarativeArc arc) {
		selection.setSelectedIndex(getInfoProvider().getArcIndexOf(arc) + getInfoProvider().getNumPlaces()
				+ getInfoProvider().getNumTrans() + 1);

	}
}
